/*  cpu.h
 *
 *  This include file contains information pertaining to the Intel
 *  80386 processor.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __CPU_h
#define __CPU_h

/* conditional compilation parameters */

#define CPU_INLINE_ENABLE_DISPATCH  TRUE
#define CPU_UNROLL_ENQUEUE_PRIORITY FALSE

/* constants */

#define CPU_MODES_INTERRUPT_LEVEL  0x00000001 /* interrupt level in mode */
#define CPU_MODES_INTERRUPT_MASK   0x00000001 /* interrupt level in mode */

#define CPU_HARDWARE_FP     TRUE    /* i80387 for i80386 */

#define CPU_CONTEXT_FP_SIZE 108     /* context size area for I80387 */
                                    /*  28 bytes for environment    */

#define CPU_SYSTEM_INITIALIZATION_THREAD_EXTRA_STACK 0

#define CPU_INTERRUPT_NUMBER_OF_VECTORS 256

#define CPU_STACK_MINIMUM_SIZE          256 

#define CPU_HEAP_PAGE_SIZE_ALIGNMENT    4      /* on 4 byte boundary */

/* macros */

/* ISR handler macros */

#define _CPU_ISR_Disable( _level ) \
  { _level = 0;\
    \
    asm volatile ( "pushf ; \
                    cli ; \
                    pop %0" \
                    : "=r" ((_level)) : "0" ((_level)) \
    ); \
  }

#define _CPU_ISR_Enable( _level )  \
  { \
    asm volatile ( "push %0 ; \
                    popf" \
                    : "=r" ((_level)) : "0" ((_level)) \
    ); \
  }

#define _CPU_ISR_Flash( _level ) \
  { \
    asm volatile ( "push %0 ; \
                    popf ; \
                    cli" \
                    : "=r" ((_level)) : "0" ((_level)) \
    ); \
  }

#define _CPU_ISR_Set_level( _new_level ) \
  { \
    if ( _new_level ) asm volatile ( "cli" ); \
    else              asm volatile ( "sti" ); \
  }

/* end of ISR handler macros */

/* Context handler macros */

#define CPU_EFLAGS_INTERRUPTS_ON  0x00003202
#define CPU_EFLAGS_INTERRUPTS_OFF 0x00003002

#define _CPU_Context_Initialize( _the_context, \
                                  _stack, _isr, _entry_point ) \
  { \
    *((proc_ptr *)(_stack)) = (_entry_point); \
    if ( (_isr) ) (_the_context)->eflags = CPU_EFLAGS_INTERRUPTS_OFF; \
    else          (_the_context)->eflags = CPU_EFLAGS_INTERRUPTS_ON; \
    (_the_context)->ebp     = (_stack); \
    (_the_context)->esp     = (_stack); \
  }

#define _CPU_Context_Restart_self( _the_context ) \
   _Context_Restore( (_the_context) );

#define _CPU_Context_Fp_start( _base, _offset ) \
   ( (void *) _Addresses_Add_offset( (_base), (_offset) ) )

#define _CPU_Context_Initialize_fp( _fp_area ) \
  { \
    unsigned32 *_source      = (unsigned32 *) _CPU_Null_fp_context; \
    unsigned32 *_destination = (unsigned32 *) *(_fp_area); \
    unsigned32  _index; \
    \
    for ( _index=0 ; _index < CPU_CONTEXT_FP_SIZE/4 ; _index++ ) \
      *_destination++ = *_source++; \
  }

/* end of Context handler macros */

/* Fatal Error manager  macros */

#define _CPU_Fatal_halt( _error ) \
  { \
    asm volatile ( "cli ; \
                    movl %0,%%eax ; \
                    hlt" \
                    : "=r" ((_error)) : "0" ((_error)) \
    ); \
  }

/* end of Fatal Error manager macros */

/* Bitfield handler macros */

#define _CPU_Bitfield_Find_first_bit( _value, _output ) \
  { \
    register unsigned16 __value_in_register = (_value); \
    \
    _output = 0; \
    \
    asm volatile ( "bsfw    %0,%1 " \
                    : "=r" (__value_in_register), "=r" (_output) \
                    : "0"  (__value_in_register), "1"  (_output) \
    ); \
  }

/* end of Bitfield handler macros */

/* Priority handler macros */

#define _CPU_Priority_Mask( _bit_number ) \
  ( 1 << (_bit_number) )

#define _CPU_Priority_Bits_index( _priority ) \
  (_priority)

/* Stack handler macros */

#define _CPU_Stack_Adjust_size( _size, _new_size ) \
   (_new_size) = (_size)

#define _CPU_Stack_Get_initial_pointer( _low_address, _size ) \
   ((_low_address) + (_size) - 4)

/* end of Stack handler macros */

/* miscellaneous */

#define _CPU_not_aligned( addr )         ((unsigned32)(addr) & 0x3)

/* structures */

/* processor defined structures */

/* See Chapter 5 - Memory Management in i386 manual */

typedef struct {
  unsigned16 limit;
  unsigned16 base_0_15;
  unsigned8  base_16_23;
  unsigned8  type_dt_dpl_p;
  unsigned8  limit_avl_g;
  unsigned8  base_24_31;
}   CPU_I80386_GDT_slot;

/* See Chapter 9 - Exceptions and Interrupts in i386 manual 
 *
 *  NOTE: This is the IDT entry for interrupt gates ONLY.
 */

typedef struct {
  unsigned16 offset_0_15;
  unsigned16 segment_selector;
  unsigned8  reserved;
  unsigned8  p_dpl;
  unsigned16 offset_16_31;
}   CPU_I80386_IDT_slot;

/* processor dependent thread structures */

typedef struct {
  unsigned32  eflags;   /* extended flags register                   */
  void       *esp;      /* extended stack pointer register           */
  void       *ebp;      /* extended base pointer register            */
  unsigned32  ebx;      /* extended bx register                      */
  unsigned32  esi;      /* extended source index register            */
  unsigned32  edi;      /* extended destination index flags register */
}   Context_Control;

typedef struct {
  unsigned32   interrupt_table_segment;
  void        *interrupt_table_offset;
}   CPU_Table;

/* variables */

EXTERN unsigned8  _CPU_Null_fp_context[ CPU_CONTEXT_FP_SIZE ];

/* functions */

void _CPU_Context_restore(
  Context_Control  *heirthread
);

void _CPU_Context_restore_fp(
  void **fp_context_ptr
);

void _CPU_Context_save_fp(
  void **fp_context_ptr
);

void _CPU_Context_switch(
  Context_Control  *run,
  Context_Control  *heir
);

void _CPU_Initialize(
  CPU_Table *cpu_table
);

void _CPU_ISR_install_vector(
  unsigned32  vector,
  proc_ptr    new_handler,
  proc_ptr   *old_handler
);

/* i80386 unique functions */

void _CPU_Install_idt(
  unsigned32 source_offset,
  unsigned16 destination_segment,
  unsigned32 destination_offset
);

/* static inline functions */

STATIC INLINE unsigned16 get_cs()
{
  register unsigned16 segment = 0;

  asm volatile ( "movw %%cs,%0" : "=r" (segment) : "0" (segment) ); 

  return segment;
}

STATIC INLINE unsigned16 get_ds()
{
  register unsigned16 segment = 0;

  asm volatile ( "movw %%ds,%0" : "=r" (segment) : "0" (segment) ); 

  return segment;
}

#endif
/* end of include file */
