/*  cpu.h
 *
 *  This include file contains information pertaining to the Motorola
 *  68020 processor.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __CPU_h
#define __CPU_h

#define NO_UNINITIALIZED_WARNINGS

/*  On some gas configurations gas needs a "%" before the register names.
 *  In particular, coff needs it and sun3 does not.  This impacts only
 *  the handful of places where a register is explicitly specified
 *  in an embedded assembly statement.  See status register related 
 *  statements and the fatal error support.
 */

#define GAS_NEEDS_PERCENT 1

/* conditional compilation parameters */

#define CPU_INLINE_ENABLE_DISPATCH   TRUE
#define CPU_UNROLL_ENQUEUE_PRIORITY  FALSE

/* constants */

#define CPU_MODES_INTERRUPT_LEVEL  0x00000007 /* interrupt level in mode */
#define CPU_MODES_INTERRUPT_MASK   0x00000007 /* interrupt level in mode */

#define CPU_HARDWARE_FP     TRUE    /* MC68881/MC68882 for MC68020       */

#define CPU_CONTEXT_FP_SIZE 332     /* context size area for MC68882     */
                                    /*   216 bytes for FSAVE/FRESTORE    */
                                    /*    96 bytes for FMOVEM FP0-7      */
                                    /*    12 bytes for FMOVEM CREGS      */
                                    /*     4 bytes for non-null flag     */

#define CPU_SYSTEM_INITIALIZATION_THREAD_EXTRA_STACK 0 

#define CPU_INTERRUPT_NUMBER_OF_VECTORS  256

#define CPU_STACK_MINIMUM_SIZE           256

#define CPU_HEAP_PAGE_SIZE_ALIGNMENT     4      /* on 4 byte boundary */

/* macros */

/* ISR handler macros */

#ifndef NO_UNINITIALIZED_WARNINGS
#define _CPU_ISR_Disable( _level ) \
  { \
    asm volatile ( "movew   %%sr,%0 ; \
                    orw     #0x0700,%%sr" \
                    : "=d" ((_level)) : "0" ((_level)) \
    ); \
  }
#else
#define _CPU_ISR_Disable( _level ) \
  { \
    (_level) = 0; \
    asm volatile ( "movew   %%sr,%0 ; \
                    orw     #0x0700,%%sr" \
                    : "=d" ((_level)) : "0" ((_level)) \
    ); \
  }
#endif

#define _CPU_ISR_Enable( _level ) \
  { \
    asm volatile ( "movew   %0,%%sr " \
                   : "=d" ((_level)) : "0" ((_level)) \
    ); \
  }

#define _CPU_ISR_Flash( _level ) \
  { \
    asm volatile ( "movew   %0,%%sr ; \
                    orw     #0x0700,%%sr" \
                    : "=d" ((_level)) : "0" ((_level)) \
    ); \
  }

#define _CPU_ISR_Set_level( _newlevel ) \
  { \
    register unsigned32 _tmpsr = 0; \
    \
    asm volatile( "movw  %%sr,%0" \
                   : "=d" (_tmpsr) : "0" (_tmpsr) \
    ); \
    \
    _tmpsr = (_tmpsr & 0xf8ff) | ((_newlevel) << 8); \
    \
    asm volatile( "movw  %0,%%sr" \
                   : "=d" (_tmpsr) : "0" (_tmpsr) \
    ); \
  }

/* end of ISR handler macros */

/* Context handler macros */

#define _CPU_Context_Initialize( _the_context, \
                                  _stack, _isr, _entry_point ) \
   { (_the_context)->sr          = 0x3000 | ((_isr) << 8); \
     (_the_context)->a7_msp      = (_stack); \
     *((proc_ptr *)(_stack))     = (_entry_point); \
   }

#define _CPU_Context_Restart_self( _the_context ) \
  { asm volatile( "movew %0,%%sr ; " \
                  "moval %1,%%a7 ; " \
                  "rts"  \
        : "=d" ((_the_context)->sr), "=d" ((_the_context)->a7_msp) \
        : "0" ((_the_context)->sr), "1" ((_the_context)->a7_msp) ); \
  }

#define _CPU_Context_Fp_start( _base, _offset ) \
   ((void *) \
     _Addresses_Add_offset( \
        (_base), \
        (_offset) + CPU_CONTEXT_FP_SIZE - 4 \
     ) \
   )

#define _CPU_Context_Initialize_fp( _fp_area ) \
   { unsigned32 *_fp_context = (unsigned32 *)*(_fp_area); \
     \
     *(--(_fp_context)) = 0; \
     *(_fp_area) = (unsigned8 *)(_fp_context); \
   }

/* end of Context handler macros */

/* Fatal Error manager macros */

#define _CPU_Fatal_halt( _error ) \
  { asm volatile( "movl  %0,%%d0; " \
                  "orw   #0x0700,%%sr; " \
                  "stop  #0x2700" : "=d" ((_error)) : "0" ((_error)) ); \
  }
      
/* end of Fatal Error manager macros */

/* Bitfield handler macros */

/*
 *  It appears that on the M68020 bitfield are always 32 bits wide
 *  when in a register.  This code forces the bitfield to be in
 *  memory (it really always is anyway). This allows us to 
 *  have a real 16 bit wide bitfield which operates "correctly."
 */

#ifndef NO_UNINITIALIZED_WARNINGS
#define _CPU_Bitfield_Find_first_bit( _value, _output ) \
  { \
    register void *__base = (void *)&(_value); \
    \
    asm volatile( "bfffo (%0),#0,#16,%1" \
                   : "=a" (__base), "=d" ((_output)) \
                   : "0"  (__base), "1" ((_output))  ) ; \
  }
#else
#define _CPU_Bitfield_Find_first_bit( _value, _output ) \
  { \
    register void *__base = (void *)&(_value); \
    \
    (_output) = 0; \
    asm volatile( "bfffo (%0),#0,#16,%1" \
                   : "=a" (__base), "=d" ((_output)) \
                   : "0"  (__base), "1" ((_output))  ) ; \
  }
#endif

/* end of Bitfield handler macros */

/* Priority handler macros */

#define _CPU_Priority_Mask( _bit_number ) \
  ( 0x8000 >> (_bit_number) )

#define _CPU_Priority_Bits_index( _priority ) \
  (_priority) 

/* end of Priority handler macros */

/* Stack handler macros */

#define _CPU_Stack_Adjust_size( _size, _new_size ) \
   (_new_size) = (_size)

#define _CPU_Stack_Get_initial_pointer( _low_address, _size ) \
   ((_low_address) + (_size) - 4)

/* end of Stack handler macros */

/* miscellaneous */

#define _CPU_not_aligned( addr )         ((unsigned32)(addr) & 0x3)

/* structures */

/* processor dependent thread structures */

typedef struct {
  unsigned32  sr;                /* (sr) status register */
  unsigned32  d2;                /* (d2) data register 2 */
  unsigned32  d3;                /* (d3) data register 3 */
  unsigned32  d4;                /* (d4) data register 4 */
  unsigned32  d5;                /* (d5) data register 5 */
  unsigned32  d6;                /* (d6) data register 6 */
  unsigned32  d7;                /* (d7) data register 7 */
  void       *a2;                /* (a2) address register 2 */
  void       *a3;                /* (a3) address register 3 */
  void       *a4;                /* (a4) address register 4 */
  void       *a5;                /* (a5) address register 5 */
  void       *a6;                /* (a6) address register 6 */
  void       *a7_msp;            /* (a7) master stack pointer */
}   Context_Control;

typedef struct {
  proc_ptr *intr_vector_tbl;
}   CPU_Table;

/* functions */

void _CPU_Context_restore(
  Context_Control  *heirthread
);

void _CPU_Context_restore_fp(
  void **fp_context_ptr
);

void _CPU_Context_save_fp(
  void **fp_context_ptr
);

void _CPU_Context_switch(
  Context_Control  *run,
  Context_Control  *heir
);

void _CPU_Initialize( 
  CPU_Table *cpu_table
);

void _CPU_ISR_install_vector( 
  unsigned32  vector,
  proc_ptr    new_handler,
  proc_ptr   *old_handler
);

#endif
/* end of include file */
