/*
 *  Clock Manager
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"
#include "clock.h"
#include "thread.h"
#include "tod.h"
#include "watchdog.h"

/*PAGE
 *
 *  _Clock_Get
 *
 *  This directive returns the current date and time.  If the time has
 *  not been set by a tm_set then an error is returned.
 *
 *  Input parameters:
 *    time_buffer - pointer to the time and date structure 
 *
 *  Output parameters:
 *    time_buffer       - pointer to time and date structure filled in
 *    STATUS_SUCCESSFUL - if successful
 *    error code        - if unsuccessful
 */

Status_Codes _Clock_Get( 
  TOD_Control *time_buffer
)
{
  if ( !_TOD_Is_set() )
    return( STATUS_NOT_DEFINED );

  *time_buffer = _TOD_Current;
  return( STATUS_SUCCESSFUL );
}

/*PAGE
 *
 *  _Clock_Set
 *
 *  This directive sets the date and time for this node.
 *
 *  Input parameters:
 *    time_buffer - pointer to the time and date structure 
 *
 *  Output parameters:
 *    STATUS_SUCCESSFUL - if successful
 *    error code        - if unsuccessful
 */

Status_Codes _Clock_Set(
  TOD_Control *time_buffer
)
{
  Status_Codes      local_result;
  Watchdog_Interval seconds;

  local_result = _TOD_Validate( time_buffer );
  if ( _Status_Is_successful( local_result ) ) {
    seconds = _TOD_To_seconds( time_buffer ); 
    _Thread_Disable_dispatch();
    _TOD_Set( time_buffer, seconds );
    _Thread_Enable_dispatch();

  }
  return( local_result );
}

/*PAGE
 *
 *  _Clock_Tick
 *
 *  This directive notifies the executve that a tick has occurred.  
 *  When the tick occurs the time manager updates and maintains
 *  the calendar time, timeslicing, and any timeout delays.
 *
 *  Input parameters:  NONE
 *
 *  Output parameters:
 *    STATUS_SUCCESSFUL - always succeeds
 *
 *  NOTE: This routine only works for leap-years through 2099.
 */

Status_Codes _Clock_Tick( void )
{
  _TOD_Tickle_ticks();

  _Watchdog_Tickle_ticks();

  _Thread_Tickle_timeslice();

  if ( _Thread_Is_context_switch_necessary() && 
       _Thread_Is_dispatching_enabled() ) 
    _Thread_Dispatch();

  return( STATUS_SUCCESSFUL );
}
