/*  config.h
 *
 *  This include file contains the table of user defined configuration
 *  parameters.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __CONFIGURATION_h
#define __CONFIGURATION_h

#include "object.h"
#include "thread.h"

/*
 *  The following records define the Initialization Tasks Table.
 *  Each entry contains the information required by RTEMS to
 *  create and start a user task automatically at executive
 *  initialization time.
 */

typedef struct {
  Objects_Name       name;              /* task name */
  unsigned32         stack_size;        /* task stack size */
  Priority_Control   initial_priority;  /* task priority */
  Attributes_Control attribute_set;     /* task attributes */
  Thread_Entry       entry_point;       /* task entry point */
  Modes_Control      mode_set;          /* task initial mode */
  unsigned32         argument;          /* task argument */
}   Configuration_Initialization_tasks_table;

/*
 *  The following records define the Device Driver Address Table.  Each
 *  entry in this table corresponds to an application provided
 *  device driver and defines the entry points for that device driver.
 */

typedef struct {
  proc_ptr initialization;       /* initialization procedure */
  proc_ptr open;                 /* open request procedure */
  proc_ptr close;                /* close request procedure */
  proc_ptr read;                 /* read request procedure */
  proc_ptr write;                /* write request procedure */
  proc_ptr control;              /* special functions procedure */
}   Configuration_Driver_address_table;

/*
 *  The following records defines the User Extension Table.
 *  This table defines the application dependent routines which
 *  are invoked at critical points in the life of each task and
 *  the system as a whole.
 */

typedef struct {
  proc_ptr task_create;            /* task create user extension */
  proc_ptr task_start;             /* task start user extension */
  proc_ptr task_restart;           /* task restart user extension */
  proc_ptr task_delete;            /* task delete user extension */
  proc_ptr task_switch;            /* task switch user extension */
  proc_ptr task_exitted;           /* task exitted user extension */
  proc_ptr fatal;                  /* fatal error user extension */
}   Configuration_Extension_table;

/*
 *  The following records define the Multiprocessor Communications
 *  Interface (MPCI) Table.  This table defines the user-provided
 *  MPCI which is a required part of a multiprocessor RTEMS system.
 */

typedef struct {
  proc_ptr initialization;         /* initialization procedure */
  proc_ptr get_packet;             /* get packet procedure */
  proc_ptr return_packet;          /* return packet procedure */
  proc_ptr send_packet;            /* packet send procedure */
  proc_ptr receive_packet;         /* packet receive procedure */
}  Configuration_MPCI_table;

/*
 *  The following records define the Multiprocessor Configuration
 *  Table.  This table defines the multiprocessor system
 *  characteristics which must be known by RTEMS in a multiprocessor
 *  system.
 */

typedef struct {
  unsigned32  node;                   /* local node number */
  unsigned32  maximum_nodes;          /* maximum # nodes in system */
  unsigned32  maximum_global_objects; /* maximum # global objects */
  unsigned32  maximum_proxies;        /* maximum # proxies */
  Configuration_MPCI_table *User_mpci_table; 
                                      /* pointer to MPCI table */
}   Configuration_Multiprocessing_table;

/*
 *  The following records define the Configuration Table.  The
 *  information contained in this table is required in all
 *  RTEMS systems, whether single or multiprocessor.  This
 *  table primarily defines the following:
 *
 *     + location and size of the RTEMS Workspace
 *     + required number of each object type
 *     + milliseconds per clock tick
 *     + clock ticks per task timeslice
 */

typedef struct {
  void                                     *work_space_start; 
  unsigned32                                work_space_size;
  unsigned32                                maximum_tasks;
  unsigned32                                maximum_timers;
  unsigned32                                maximum_semaphores;
  unsigned32                                maximum_message_queues;
  unsigned32                                maximum_messages;
  unsigned32                                maximum_partitions;
  unsigned32                                maximum_regions;
  unsigned32                                maximum_ports;
  unsigned32                                maximum_periods;
  unsigned32                                milliseconds_per_tick;
  unsigned32                                ticks_per_timeslice;
  unsigned32                                number_of_initialization_tasks;
  Configuration_Initialization_tasks_table *User_initialization_tasks_table;
  unsigned32                                number_of_device_drivers;
  Configuration_Driver_address_table       *Device_driver_table; 
  Configuration_Extension_table            *User_extension_table;
  Configuration_Multiprocessing_table      *User_multiprocessing_table;
}   Configuration_Table;

/*
 *  The following defines the default Multiprocessing Configuration
 *  Table.  This table is used in a single processor system.
 */

extern const Configuration_Multiprocessing_table  
  _Configuration_Default_multiprocessing_table;

/*
 *  The following define the internal pointers to the user's
 *  configuration information.
 */

EXTERN Configuration_Table                 *_Configuration_Table; 
EXTERN Configuration_Multiprocessing_table *_Configuration_MP_table;
EXTERN Configuration_MPCI_table            *_Configuration_MPCI_table;

/*
 *
 *  _Configuration_Handler_initialization
 *
 *  This routine performs the initialization necessary for this handler.
 */

STATIC INLINE void _Configuration_Handler_initialization(
  Configuration_Table                 *configuration_table,
  Configuration_Multiprocessing_table *multiprocessing_table,
  Configuration_MPCI_table            *users_mpci_table
);

/*
 *  _Configuration_Is_multiprocessing
 *
 *  This function determines if a multiprocessing application has been
 *  configured, if so, TRUE is returned, otherwise FALSE is returned.
 */

STATIC INLINE boolean _Configuration_Is_multiprocessing( void );

/*
 *  _Configuration_Is_null_driver_address_table_pointer
 *
 *  This function returns TRUE if the_table is NULL and FALSE otherwise.
 */

STATIC INLINE boolean _Configuration_Is_null_driver_address_table_pointer(
  Configuration_Driver_address_table *the_table
);

/*
 *  _Configuration_Is_null_extension_table_pointer
 *
 *  This function returns TRUE if the_table is NULL and FALSE otherwise.
 */

STATIC INLINE boolean _Configuration_Is_null_extension_table_pointer( 
  Configuration_Extension_table *the_table
);

/*
 *  _Configuration_Is_null_initialization_tasks_table_pointer
 *
 *  This function returns TRUE if the_table is NULL and FALSE otherwise.
 */

STATIC INLINE boolean
    _Configuration_Is_null_initialization_tasks_table_pointer(
  Configuration_Initialization_tasks_table *the_table
);

#include "config.inl"

#endif
/* end of include file */
