/*
 *  Initialization Manager
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

/*
 *  INIT is defined so all of the data will be included in this
 *  file.
 */

#define INIT

#include "system.h"
#include "config.h"
#include "copyrt.h"
#include "clock.h"
#include "tasks.h"
#include "dpmem.h"
#include "event.h"
#include "fatal.h"
#include "heap.h"
#include "init.h"
#include "intthrd.h"
#include "isr.h"
#include "intr.h"
#include "io.h"
#include "message.h"
#include "mp.h"
#include "mpci.h"
#include "part.h"
#include "priority.h"
#include "ratemon.h"
#include "region.h"
#include "sem.h"
#include "signal.h"
#include "thread.h"
#include "timer.h"
#include "tod.h"
#include "userext.h"
#include "watchdog.h"
#include "wkspace.h"

#include "cputable.h"
#include "sptables.h"

/*PAGE
 *
 *  _Initialize_Executive
 * 
 *  This directive initializes all the kernels data structures
 *  to the states necessary for the kernel to begin execution.  All
 *  include files that contain global variable definitions should be
 *  included in this file.  The system threads and initialization threads
 *  are created and started by this routine.  This routine then  
 *  initiates multithreading.
 *
 *  Input parameters:
 *    configuration_table - pointer to the user's configuration table
 *    cpu_table           - pointer to the user's CPU configuration table
 * 
 *  Output parameters:  NONE
 */

struct months {
  unsigned32 months[2][13];
};

void _Initialize_Executive( 
  Configuration_Table *configuration_table,
  CPU_Table           *cpu_table
)
{
  ISR_Level                            bsp_level;
  Configuration_Multiprocessing_table *multiprocessing_table;

  /* Dispatching and interrupts are disabled until the end of the 
   * initialization sequence.  This prevents an inadvertent context 
   * switch before the executive is initialized.
   */
  
  _ISR_Disable( bsp_level );

  _CPU_Initialize( cpu_table );

  multiprocessing_table = configuration_table->User_multiprocessing_table;

  if ( multiprocessing_table == NULL )
    multiprocessing_table = 
      (void *) &_Configuration_Default_multiprocessing_table;

  _Configuration_Handler_initialization(
    configuration_table,
    multiprocessing_table,
    multiprocessing_table->User_mpci_table
  );

  _Thread_Dispatch_initialization();

  _User_extensions_Handler_initialization(
     configuration_table->User_extension_table
  );

  _Workspace_Handler_initialization( 
     (void *)configuration_table->work_space_start, 
     configuration_table->work_space_size 
  );

  _ISR_Handler_initialization();

  _Objects_Handler_initialization(
    multiprocessing_table->node,
    multiprocessing_table->maximum_global_objects
  );

  _Priority_Handler_initialization();

  _Watchdog_Handler_initialization();

  _TOD_Handler_initialization( configuration_table->milliseconds_per_tick );

  _Thread_Handler_initialization( 
    configuration_table->maximum_tasks,
    configuration_table->ticks_per_timeslice,
    multiprocessing_table->maximum_proxies
  );

  _MPCI_Handler_initialization();

/* MANAGERS */

  _Multiprocessing_Manager_initialization();

  _Timer_Manager_initialization( configuration_table->maximum_timers );

  _IO_Manager_initialization(
      configuration_table->Device_driver_table,
      configuration_table->number_of_device_drivers
  );

  _Event_Manager_initialization();

  _Message_queue_Manager_initialization( 
    configuration_table->maximum_message_queues,
    configuration_table->maximum_messages
  );

  _Semaphore_Manager_initialization(
    configuration_table->maximum_semaphores
  );

  _Partition_Manager_initialization(
    configuration_table->maximum_partitions
  );

  _Region_Manager_initialization( configuration_table->maximum_regions );

  _Dual_ported_memory_Manager_initialization(
    configuration_table->maximum_ports
  );

  _Rate_monotonic_Manager_initialization( 
    configuration_table->maximum_periods
  );

  _Internal_threads_Initialization();

  _Thread_Start_multitasking( 
    _Internal_threads_System_initialization_thread,
    _Internal_threads_Idle_thread
  );

}

/*PAGE
 *
 *  _Shutdown_Executive
 * 
 *  This kernel routine shutdowns the executive.  It halts multitasking
 *  and returns control to the application execution "thread" which
 *  initialially invoked the initialize_executive directive.
 *
 *  Input parameters:   NONE
 * 
 *  Output parameters:  NONE
 */

#include "system.h"
#include "init.h"
#include "thread.h"

void _Shutdown_Executive( 
   unsigned32 result
)
{
  _Thread_Stop_multitasking();
}
