/*
 *  Internal Threads
 *
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"
#include "config.h"
#include "fatal.h"
#include "intthrd.h"
#include "io.h"
#include "mpci.h"
#include "object.h"
#include "states.h"
#include "tasks.h"
#include "thread.h"
#include "wkspace.h"

/*PAGE
 *
 *  _Internal_threads_Set_MP_receive_server
 *
 *  This subprogram informs the system initialization thread 
 *  of the entry point of the multiprocessing server. 
 *
 *  Input parameters: 
 *    mp_server_entry - address of the multiprocessing receive server task
 *
 *  Output parameters:  NONE
 */

void _Internal_threads_Set_MP_receive_server( 
  proc_ptr mp_server_entry
)
{
  _Internal_threads_MP_Receive_server_entry = mp_server_entry;
}

/*PAGE
 *
 *  _Internal_threads_Initialization
 *
 *  This kernel routine creates and starts the system initialization and
 *  idle threads.  The priorities for the two threads must be adjusted to
 *  priority 0 (an illegal priority) to insure that no application thread
 *  runs until system initialization is complete.
 *
 *  Input parameters:   NONE
 *
 *  Output parameters:  NONE
 */

void _Internal_threads_Initialization( void )
{
  /* all fields not explicitly assigned are zeroed by 
   * _Workspace initialization
   */

  _Workspace_Allocate_or_fatal_error(
    sizeof(Thread_Control),
    (void **) &_Internal_threads_System_initialization_thread
  );

  _Workspace_Allocate_or_fatal_error(
    sizeof(Thread_Control),
    (void **) &_Internal_threads_Idle_thread
  );

  _Internal_threads_Initialize_thread(
    _Internal_threads_System_initialization_thread,
    _Objects_Build_name( 'S', 'Y', 'S', 'I' ),
    PRIORITY_MINIMUM - 1,
    ATTRIBUTES_DEFAULTS,
    MODES_NO_PREEMPT,
    _Internal_threads_System_initialization_thread_body,
    INTERNAL_THREADS_SYSTEM_INITIALIZATION_THREAD_STACK_SIZE
  );
    
  _Internal_threads_Initialize_thread(
    _Internal_threads_Idle_thread,
    _Objects_Build_name( 'I', 'D', 'L', 'E' ),
    PRIORITY_MINIMUM - 1,
    ATTRIBUTES_DEFAULTS,
    MODES_DEFAULTS,
    _Internal_threads_Idle_thread_body,
    INTERNAL_THREADS_IDLE_THREAD_STACK_SIZE
  );
    
}

/*PAGE
 *
 *  _Internal_threads_Initialize_user_tasks
 * 
 *  This routine creates and starts all configured user 
 *  initialzation threads.
 *
 *  Input parameters: NONE
 * 
 *  Output parameters:  NONE
 */

void _Internal_threads_Initialize_user_tasks( void )
{
  unsigned32   index;
  Objects_Id   id;
  Status_Codes return_value;
  Configuration_Initialization_tasks_table *user_tasks;

  user_tasks = _Configuration_Table->User_initialization_tasks_table;

  if ( user_tasks == NULL || 
       _Configuration_Table->number_of_initialization_tasks == 0 ) 
    _Fatal_Error_occurred( STATUS_INVALID_NUMBER );

  for ( index=0 ; 
        index < _Configuration_Table->number_of_initialization_tasks ; 
        index++ ) {

    return_value = _RTEMS_tasks_Create( 
      user_tasks[ index ].name, 
      user_tasks[ index ].initial_priority, 
      user_tasks[ index ].stack_size,
      user_tasks[ index ].mode_set, 
      user_tasks[ index ].attribute_set, 
      &id 
    );

    if ( !_Status_Is_successful( return_value ) ) 
      _Fatal_Error_occurred( return_value );

    return_value = _RTEMS_tasks_Start( 
      id,
      user_tasks[ index ].entry_point, 
      user_tasks[ index ].argument
    );

    if ( !_Status_Is_successful( return_value ) ) 
      _Fatal_Error_occurred( return_value );
  }
}

/*PAGE
 *
 *  _Internal_threads_Idle_thread_body
 *
 *  This kernel routine is the idle thread.  The idle thread runs any time
 *  no other thread is ready to run.  This thread loops forever with 
 *  interrupts enabled.
 *
 *  Input parameters: 
 *    ignored - this parameter is ignored
 *
 *  Output parameters:  NONE
 */

Thread _Internal_threads_Idle_thread_body( 
  unsigned32 ignored
)
{
  while( FOREVER ) ;
}

/*PAGE
 *
 *  _Internal_threads_System_initialization_thread_body
 *
 *  This kernel routine is the system initialization thread.  
 *  It is the first thread to execute because it performs 
 *  system initialization functions such as calling 
 *  _IO_initialization for each of the application's drivers.
 *  After initialization if multiprocessing is enabled the the thread
 *  becomes the multiprocessing receive thread, otherwise 
 *  (i.e. in a single processor system) the thread kills itself.
 *
 *  Input parameters:
 *    ignored - ignores the parameter all threads receive
 *
 *  Output parameters:  NONE
 */

Thread _Internal_threads_System_initialization_thread_body(
  unsigned32 ignored
)
{
  _IO_Initialize_all_drivers();

  if ( _Configuration_Is_multiprocessing() ) {
    _MPCI_Initialization();
    _Internal_threads_MP_Send_process_packet(
      INTERNAL_THREADS_MP_SYSTEM_VERIFY
    );
  }

  _Thread_Change_priority( 
    _Internal_threads_Idle_thread, 
    PRIORITY_MAXIMUM
  );

  _Internal_threads_Initialize_user_tasks();

  if ( _Configuration_Is_multiprocessing() )
    (*_Internal_threads_MP_Receive_server_entry)(); /* DOES NOT RETURN */

  _Thread_Disable_dispatch();               /* SIMPLE DELETE OF SELF */
  _Thread_Set_state( _Thread_Executing, STATES_TRANSIENT );
  _Thread_Enable_dispatch();
}

/*PAGE
 *
 *  _Internal_threads_Initialize_thread
 *
 *  This routine initializes the specified system thread.
 *
 *  Input parameters:
 *    the_thread     - pointer to the internal thread to be initialized
 *    its_name       - the name of the internal thread
 *    its_priority   - the initial priority of the internal thread
 *    its_attributes - the initial attributes of the internal thread
 *    its_mode       - the initial mode of the internal thread
 *    entry_point    - address of the code for the internal thread
 *    stack_size     - the initial stack size for the internal thread
 *
 *  Output parameters:  NONE
 */

void _Internal_threads_Initialize_thread(
  Thread_Control     *the_thread,
  Objects_Name        its_name,
  Priority_Control    its_priority,
  Attributes_Control  its_attributes,
  Modes_Control       its_mode,
  proc_ptr            entry_point,
  unsigned32          stack_size
)
{
  void *memory; 
 
  _Workspace_Allocate_or_fatal_error( stack_size, &memory );
 
  _Stack_Initialize( &the_thread->Start.Initial_stack, memory, stack_size );
  the_thread->name                   = its_name;
  the_thread->attribute_set          = its_attributes;
  the_thread->current_state          = STATES_DORMANT;
  the_thread->Start.initial_modes    = its_mode;
  the_thread->current_modes          = its_mode;
  the_thread->Start.entry_point      = entry_point;
  the_thread->Start.initial_argument = 0;
  the_thread->Start.fp_context       = NULL;
  the_thread->fp_context             = NULL;

  _Thread_Set_priority( the_thread, its_priority );

  _Thread_Load_environment( the_thread );
}
