/*
 *  Input/Output Manager
 *
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"
#include "config.h"
#include "io.h"
#include "isr.h"
#include "thread.h"

/*PAGE
 *
 *  _IO_Initialize_all_drivers
 *
 *  This routine initializes all device drivers
 *
 *  Input Paramters:   NONE
 *
 *  Output Parameters: NONE
 */

void _IO_Initialize_all_drivers( void )
{
   IO_Major_control major;
   unsigned32       return_value;

   for ( major=0 ; major < _IO_Number_of_drivers ; major ++ )
     (void) _IO_Initialize( major, 0, _Configuration_Table, &return_value );
}

/*PAGE
 *
 *  _IO_Initialize
 *
 *  This routine is the initialization directive of the IO manager.
 *
 *  Input Paramters:
 *    major        - device driver number
 *    minor        - device number 
 *    argument     - pointer to argument(s)
 *    return_value - pointer to driver's return value 
 *
 *  Output Parameters:
 *    returns       - return code
 *    *return_value - driver's return code
 */

Status_Codes _IO_Initialize( 
  IO_Major_control  major,
  IO_Minor_control  minor,
  void             *argument,
  unsigned32       *return_value
)
{
  return _IO_Handler_routine(
            IO_INITIALIZE_OPERATION,
            major, 
            minor,
            argument,
            return_value
         );
}

/*PAGE
 *
 *  _IO_Open
 *
 *  This routine is the open directive of the IO manager.
 *
 *  Input Paramters:
 *    major        - device driver number
 *    minor        - device number 
 *    argument     - pointer to argument(s)
 *    return_value - pointer to driver's return value 
 *
 *  Output Parameters:
 *    returns       - return code
 *    *return_value - driver's return code
 */

Status_Codes _IO_Open( 
  IO_Major_control  major,
  IO_Minor_control  minor,
  void             *argument,
  unsigned32       *return_value
)
{
  return _IO_Handler_routine(
            IO_OPEN_OPERATION,
            major, 
            minor,
            argument,
            return_value
         );
}

/*PAGE
 *
 *  _IO_Close
 *
 *  This routine is the close directive of the IO manager.
 *
 *  Input Paramters:
 *    major        - device driver number
 *    minor        - device number 
 *    argument     - pointer to argument(s)
 *    return_value - pointer to driver's return value 
 *
 *  Output Parameters:
 *    returns       - return code
 *    *return_value - driver's return code
 */

Status_Codes _IO_Close(
  IO_Major_control  major,
  IO_Minor_control  minor,
  void             *argument,
  unsigned32       *return_value
)
{
  return _IO_Handler_routine(
            IO_CLOSE_OPERATION,
            major, 
            minor,
            argument,
            return_value
         );
}

/*PAGE
 *
 *  _IO_Read
 *
 *  This routine is the read directive of the IO manager.
 *
 *  Input Paramters:
 *    major        - device driver number
 *    minor        - device number 
 *    argument     - pointer to argument(s)
 *    return_value - pointer to driver's return value 
 *
 *  Output Parameters:
 *    returns       - return code
 *    *return_value - driver's return code
 */

Status_Codes _IO_Read( 
  IO_Major_control  major,
  IO_Minor_control  minor,
  void             *argument,
  unsigned32       *return_value
)
{
  return _IO_Handler_routine(
            IO_READ_OPERATION,
            major, 
            minor,
            argument,
            return_value
         );
}

/*PAGE
 *
 *  _IO_Write
 *
 *  This routine is the write directive of the IO manager.
 *
 *  Input Paramters:
 *    major        - device driver number
 *    minor        - device number 
 *    argument     - pointer to argument(s)
 *    return_value - pointer to driver's return value 
 *
 *  Output Parameters:
 *    returns       - return code
 *    *return_value - driver's return code
 */

Status_Codes _IO_Write( 
  IO_Major_control  major,
  IO_Minor_control  minor,
  void             *argument,
  unsigned32       *return_value
)
{
  return _IO_Handler_routine(
            IO_WRITE_OPERATION,
            major, 
            minor,
            argument,
            return_value
         );
}

/*PAGE
 *
 *  _IO_Control
 *
 *  This routine is the control directive of the IO manager.
 *
 *  Input Paramters:
 *    major        - device driver number
 *    minor        - device number 
 *    argument     - pointer to argument(s)
 *    return_value - pointer to driver's return value 
 *
 *  Output Parameters:
 *    returns       - return code
 *    *return_value - driver's return code
 */

Status_Codes _IO_Control( 
  IO_Major_control  major,
  IO_Minor_control  minor,
  void             *argument,
  unsigned32       *return_value
)
{
  return _IO_Handler_routine(
            IO_CONTROL_OPERATION,
            major, 
            minor,
            argument,
            return_value
         );
}

/*PAGE
 *
 *  _IO_Handler_routine
 *
 *  This routine implements all IO manager directives.
 *
 *  Input Paramters:
 *    operation    - I/O operation to be performed
 *    major        - device driver number
 *    minor        - device number 
 *    argument     - pointer to argument(s)
 *    return_value - pointer to driver's return value 
 *
 *  Output Parameters:
 *    returns       - return code
 *    *return_value - driver's return code
 */

Status_Codes _IO_Handler_routine( 
  IO_operations     operation,
  IO_Major_control  major,
  IO_Minor_control  minor,
  void             *argument,
  unsigned32       *return_value
)
{
  proc_ptr io_callout; 
 
  if ( major >= _IO_Number_of_drivers ) 
    return ( STATUS_INVALID_NUMBER );

  switch ( operation ) {
     case IO_INITIALIZE_OPERATION:
        io_callout = _IO_Driver_address_table[ major ].initialization;
        break;
     case IO_OPEN_OPERATION:
        io_callout = _IO_Driver_address_table[ major ].open;
        break;
     case IO_CLOSE_OPERATION:
        io_callout = _IO_Driver_address_table[ major ].close;
        break;
     case IO_READ_OPERATION:
        io_callout = _IO_Driver_address_table[ major ].read;
        break;
     case IO_WRITE_OPERATION:
        io_callout = _IO_Driver_address_table[ major ].write;
        break;
     case IO_CONTROL_OPERATION:
        io_callout = _IO_Driver_address_table[ major ].control;
        break;
     default:             /* unreached -- only to remove warnings */
        io_callout = NULL;
        break;
  }

  if ( io_callout != NULL ) 
     (*io_callout)( 
        major,
        minor, 
        argument, 
        _Thread_Executing->Object.id, 
        return_value 
     ); 
  else    
      *return_value = 0; 

  return( STATUS_SUCCESSFUL ); 
}
