/*  io.h
 * 
 *  This include file contains all the constants and structures associated
 *  with the Input/Output Manager.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __IO_h
#define __IO_h

#include "config.h"

/*
 *  The following types define the control structures required to
 *  manage device driver major and minor numbers.
 */

typedef unsigned32 IO_Major_control;
typedef unsigned32 IO_Minor_control;

/*
 *  The following type defines the set of IO operations which are
 *  recognized by _IO_Handler and can be supported by a RTEMS
 *  device driver.
 */

typedef enum {
  IO_INITIALIZE_OPERATION = 0,
  IO_OPEN_OPERATION       = 1,
  IO_CLOSE_OPERATION      = 2,
  IO_READ_OPERATION       = 3,
  IO_WRITE_OPERATION      = 4,
  IO_CONTROL_OPERATION    = 5
}  IO_operations;

/*
 *  The following declare the data required to manage the Device Driver
 *  Address Table.
 */

EXTERN unsigned32                          _IO_Number_of_drivers;
EXTERN Configuration_Driver_address_table *_IO_Driver_address_table;

/*
 *  _IO_Manager_initialization
 *
 *  This routine performs the initialization necessary for this manager.
 */

STATIC INLINE void _IO_Manager_initialization(
  Configuration_Driver_address_table *driver_table,
  unsigned32                          number_of_drivers
);

/*
 *  _IO_Initialize
 *
 *  This routine implements the io_initialize directive.  It is invoked
 *  to initialize a device driver or an individual device.
 */

Status_Codes _IO_Initialize( 
  IO_Major_control  major,
  IO_Minor_control  minor, 
  void             *argument, 
  unsigned32       *return_value 
);

/*
 *  _IO_Open
 *
 *  This routine implements the io_open directive.  It is invoked
 *  to open a device.
 */

Status_Codes _IO_Open(
  IO_Major_control  major,
  IO_Minor_control  minor, 
  void             *argument, 
  unsigned32       *return_value 
);

/*
 *  _IO_Close
 *
 *  This routine implements the io_close directive.  It is invoked
 *  to close a device.
 */

Status_Codes _IO_Close(
  IO_Major_control  major,
  IO_Minor_control  minor, 
  void             *argument, 
  unsigned32       *return_value 
);

/*
 *  _IO_Read
 *
 *  This routine implements the io_read directive.  It is invoked
 *  to read from a device.
 */

Status_Codes _IO_Read(
  IO_Major_control  major,
  IO_Minor_control  minor, 
  void             *argument, 
  unsigned32       *return_value 
);

/*
 *  _IO_Write
 *
 *  This routine implements the io_write directive.  It is invoked
 *  to write to a device.
 */

Status_Codes _IO_Write(
  IO_Major_control  major,
  IO_Minor_control  minor, 
  void             *argument, 
  unsigned32       *return_value 
);

/*
 *  _IO_Control
 *
 *  This routine implements the io_control directive.  It is invoked
 *  to perform a device specific operation on a device.
 */

Status_Codes _IO_Control(
  IO_Major_control  major,
  IO_Minor_control  minor, 
  void             *argument, 
  unsigned32       *return_value 
);

/*
 *  _IO_Initialize_all_drivers
 *
 *  This routine initializes all of the device drivers configured
 *  in the Device Driver Address Table.
 */

void _IO_Initialize_all_drivers( void );

/*
 *  _IO_Handler_routine
 *
 *  This routine provides the common foundation for all of the IO
 *  Manager's directives.
 */

Status_Codes _IO_Handler_routine(
  IO_operations     operation,
  IO_Major_control  major,
  IO_Minor_control  minor, 
  void             *argument, 
  unsigned32       *return_value
);

#include "io.inl"

#endif
/* end of include file */
