/*  object.h
 * 
 *  This include file contains all the constants and structures associated
 *  with the RTEMS Object Handler.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __OBJECTS_h
#define __OBJECTS_h

#include "chain.h"

/*
 *  The following type defines the control block used to manage
 *  object names.
 */

typedef unsigned32 Objects_Name;

/*
 *  The following type defines the control block used to manage
 *  object IDs.
 */

typedef unsigned32 Objects_Id;

/*
 *  This enumerated type lists the locations which may be returned
 *  by _Objects_Get.  These codes indicate the success of locating
 *  an object with the specified ID.
 */

typedef enum {
  OBJECTS_LOCAL  = 0,         /* object is local */
  OBJECTS_REMOTE = 1,         /* object is remote */
  OBJECTS_ERROR  = 2          /* id was invalid */
}  Objects_Locations;

/*
 *  The following defines the Object Control Block used to manage
 *  each object local to this node.
 */

typedef struct {
  Chain_Node Node; 
  Objects_Id id;
}   Objects_Control;

/*
 *  The following defines the structure for the information used to
 *  manage each class of objects.
 */

typedef struct {
  Objects_Id        minimum_id;      /* minimum valid id of this type */
  Objects_Id        maximum_id;      /* maximum valid id of this type */
  unsigned32        maximum;         /* maximum number of objects */
  Objects_Control **local_table;     /* table of local object pointers */
  Objects_Name     *name_table;      /* table of local object names */
  Chain_Control    *global_table;    /* pointer to global table */
  Chain_Control     Inactive;        /* chain of inactive ctl blocks */
}   Objects_Information;

/*
 *  The following defines the data storage which contains the
 *  node number of the local node.
 */

EXTERN unsigned32  _Objects_Local_node;

/*
 *  The following define the constants which may be used in name searches.
 */

#define OBJECTS_SEARCH_ALL_NODES   0
#define OBJECTS_SEARCH_OTHER_NODES 0x7FFFFFFE
#define OBJECTS_SEARCH_LOCAL_NODE  0x7FFFFFFF
#define OBJECTS_WHO_AM_I           0

/*
 *  _Objects_Handler_initialization
 *
 */

void _Objects_Handler_initialization(
  unsigned32 node,
  unsigned32 maximum_global_objects
);

/*
 *  _Objects_Initialize_information
 *
 */

void _Objects_Initialize_information (
  Objects_Information *information,
  boolean              supports_global,
  unsigned32           maximum,
  unsigned32           size
);

/*
 *  _Objects_Name_to_id
 *
 */

Status_Codes _Objects_Name_to_id( 
  Objects_Information *information,
  Objects_Name         name, 
  unsigned32           node, 
  Objects_Id          *id 
);

/*
 *  _Objects_Get
 *
 */

Objects_Control *_Objects_Get ( 
  Objects_Information *information,
  Objects_Id           id,
  Objects_Locations   *location
);

/*
 *  _Objects_Is_name_valid
 *
 */

STATIC INLINE boolean _Objects_Is_name_valid ( 
  Objects_Name name
);

/*
 *  _Objects_Build_name
 *
 *  NOTE:
 *
 *  This must be implemented as a macro for use in Configuration Tables.
 */

#define _Objects_Build_name( _C1, _C2, _C3, _C4 ) \
  ( (_C1) << 24 | (_C2) << 16 | (_C3) << 8 | (_C4) )

/*
 *  _Objects_Build_id
 *
 */

STATIC INLINE Objects_Id _Objects_Build_id( 
  unsigned32 node, 
  unsigned32 index
); 

/*
 *  _Objects_Get_node
 *
 */

STATIC INLINE unsigned32 _Objects_Get_node(
  Objects_Id id
);

/*
 *  _Objects_Get_index
 *
 */

STATIC INLINE unsigned32 _Objects_Get_index(
  Objects_Id id
);

/*
 *  _Objects_Is_local_node
 *
 */

STATIC INLINE boolean _Objects_Is_local_node(
  unsigned32 node
);

/*
 *  _Objects_Is_local_id
 *
 */

STATIC INLINE boolean _Objects_Is_local_id( 
  Objects_Id id
);

/*
 *  _Objects_Are_ids_equal
 *
 */

STATIC INLINE boolean _Objects_Are_ids_equal( 
  Objects_Id left, 
  Objects_Id right
);

/*
 *  _Objects_Allocate
 *
 */

STATIC INLINE Objects_Control *_Objects_Allocate( 
  Objects_Information *information
);

/*
 *  _Objects_Free
 *
 */

STATIC INLINE void _Objects_Free(
  Objects_Information *information,
  Objects_Control     *the_object
);

/*
 *  _Objects_Open
 *
 */

STATIC INLINE void _Objects_Open(
  Objects_Information *information,
  Objects_Control     *the_object,
  Objects_Name         name 
);

/*
 *  _Objects_Close
 *
 */

STATIC INLINE void _Objects_Close(
  Objects_Information *information,
  Objects_Control     *the_object
);

#include "object.inl"
#include "objectmp.h"

#endif
/* end of include file */
