/*
 *  Region Manager 
 *
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"
#include "config.h"
#include "object.h"
#include "options.h"
#include "region.h"
#include "states.h"
#include "thread.h"

/*PAGE
 * 
 *  _Region_Manager_initialization
 * 
 *  This routine initializes all region manager related data structures.
 *
 *  Input parameters:
 *    maximum_regions - number of regions to initialize
 * 
 *  Output parameters:  NONE
 */

void _Region_Manager_initialization( 
  unsigned32 maximum_regions
)
{
  _Objects_Initialize_information( 
     &_Region_Information, 
     FALSE,
     maximum_regions, 
     sizeof( Region_Control )
   );
}

/*PAGE
 * 
 *  _Region_Create
 *
 *  This directive creates a region of physical contiguous memory area
 *  from which variable sized segments can be allocated.
 *
 *  Input parameters:
 *    name             - user defined region name
 *    starting_address - physical start address of region
 *    length           - physical length in bytes
 *    page_size        - page size in bytes
 *    attribute_set    - region attributes
 *    id               - address of region id to set
 *
 *  Output parameters:
 *    id       - region id
 *    STATUS_SUCCESSFUL - if successful
 *    error code - if unsuccessful
 */

Status_Codes _Region_Create( 
  Objects_Name        name,
  void               *starting_address,
  unsigned32          length,
  unsigned32          page_size,
  Attributes_Control  attribute_set,
  Objects_Id         *id
)
{
  Region_Control *the_region;

  if ( !_Objects_Is_name_valid( name ) ) 
    return ( STATUS_INVALID_NAME );

  if ( !_Addresses_Is_aligned( starting_address ) )
    return( STATUS_INVALID_ADDRESS );

  _Thread_Disable_dispatch();             /* to prevent deletion */
  if ( !(the_region = _Region_Allocate()) ) {
    _Thread_Enable_dispatch();
    return( STATUS_TOO_MANY );
  }
  
  if ( !(the_region->maximum_segment_size = _Heap_Initialize( 
           &the_region->Memory, starting_address, length, page_size )) ) {
    _Region_Free( the_region );
    _Thread_Enable_dispatch();
    return( STATUS_INVALID_SIZE );
  }

  the_region->starting_address      = starting_address;
  the_region->length                = length;
  the_region->page_size             = page_size;
  the_region->attribute_set         = attribute_set;
  the_region->number_of_used_blocks = 0;

  _Thread_queue_Initialize( 
     &the_region->Wait_queue, attribute_set, STATES_WAITING_FOR_SEGMENT );

  _Objects_Open( &_Region_Information, &the_region->Object, name );

  *id = the_region->Object.id;
  _Thread_Enable_dispatch();
  return( STATUS_SUCCESSFUL );
}

/*PAGE
 * 
 *  _Region_Name_to_id
 *
 *  This directive returns the system ID associated with
 *  the region name.
 *
 *  Input parameters:
 *    name - user defined region name
 *    id   - pointer to region id
 *
 *  Output parameters:
 *    *id      - region id
 *    STATUS_SUCCESSFUL - if successful
 *    error code - if unsuccessful
 */

Status_Codes _Region_Name_to_id( 
  Objects_Name  name,
  Objects_Id   *id
)
{
  return( _Objects_Name_to_id( &_Region_Information, name, 0, id ) );
}

/*PAGE
 * 
 *  _Region_Delete
 *
 *  This directive allows a thread to delete a region specified by
 *  the region identifier, provided that none of its segments are
 *  still allocated.
 *
 *  Input parameters:
 *    id - region id
 *
 *  Output parameters:
 *    STATUS_SUCCESSFUL - if successful
 *    error code - if unsuccessful
 */

Status_Codes _Region_Delete( 
  Objects_Id id
)
{
  register Region_Control *the_region;
  Objects_Locations               location;

  the_region = _Region_Get( id, &location );
  switch ( location ) {
    case OBJECTS_ERROR:
    case OBJECTS_REMOTE:        /* this error cannot be returned */
      return( STATUS_INVALID_ID );
    case OBJECTS_LOCAL:
      if ( the_region->number_of_used_blocks == 0 ) {
        _Objects_Close( &_Region_Information, &the_region->Object );
        _Region_Free( the_region );
        _Thread_Enable_dispatch();
        return( STATUS_SUCCESSFUL );
      }
      _Thread_Enable_dispatch();
      return( STATUS_RESOURCE_IN_USE );
  }

  return( STATUS_SUCCESSFUL );   /* unreached - only to remove warnings */
}

/*PAGE
 * 
 *  _Region_Get_segment
 *
 *  This directive will obtain a segment from the given region.
 *
 *  Input parameters:
 *    id         - region id
 *    size       - segment size in bytes
 *    option_set - wait option
 *    timeout    - number of ticks to wait (0 means wait forever)
 *    segment    - pointer to segment address
 *
 *  Output parameters:
 *    segment    - pointer to segment address filled in
 *    STATUS_SUCCESSFUL - if successful
 *    error code - if unsuccessful
 */

Status_Codes _Region_Get_segment( 
  Objects_Id         id,
  unsigned32         size,
  Options_Control    option_set,
  Watchdog_Interval  timeout,
  void              **segment
)
{
  register Region_Control *the_region;
  Objects_Locations               location;
  Thread_Control          *executing;

  executing  = _Thread_Executing; 
  the_region = _Region_Get( id, &location );
  switch ( location ) {
    case OBJECTS_ERROR:
    case OBJECTS_REMOTE:        /* this error cannot be returned */
      return( STATUS_INVALID_ID );
    case OBJECTS_LOCAL:
      if ( size > the_region->maximum_segment_size ) {
        _Thread_Enable_dispatch();
        return( STATUS_INVALID_SIZE );
      }

      if ( (*segment = _Region_Allocate_segment( the_region, size )) ) {
        the_region->number_of_used_blocks += 1;
        _Thread_Enable_dispatch();
        return( STATUS_SUCCESSFUL );
      }

      if ( _Options_Is_no_wait( option_set ) ) {
        _Thread_Enable_dispatch();
        return( STATUS_UNSATISFIED );
      }

      executing->Wait.queue              = &the_region->Wait_queue;
      executing->Wait.id                 = id;
      executing->Wait.Extra.segment_size = size;
      executing->Wait.return_argument    = (unsigned32 *) segment;

      the_region->Wait_queue.sync = TRUE;

      _Thread_queue_Enqueue( &the_region->Wait_queue, timeout );

      _Thread_Enable_dispatch();
      return( executing->Wait.return_code );
  }

  return( STATUS_SUCCESSFUL );   /* unreached - only to remove warnings */
}

/*PAGE
 * 
 *  _Region_Return_segment
 *
 *  This directive will return a segment to its region.
 *
 *  Input parameters:
 *    id      - region id
 *    segment - pointer to segment address
 *
 *  Output parameters:
 *    STATUS_SUCCESSFUL - if successful
 *    error code        - if unsuccessful
 */

Status_Codes _Region_Return_segment( 
  Objects_Id  id,
  void       *segment
)
{
  register Region_Control *the_region;
  Thread_Control          *the_thread;
  Objects_Locations               location;
  void                           **the_segment;

  the_region = _Region_Get( id, &location );
  switch ( location ) {
    case OBJECTS_ERROR:
    case OBJECTS_REMOTE:        /* this error cannot be returned */
      return( STATUS_INVALID_ID );
    case OBJECTS_LOCAL:
      if ( !_Region_Free_segment( the_region, segment ) ) {
        _Thread_Enable_dispatch();
        return( STATUS_INVALID_ADDRESS );
      }

      the_region->number_of_used_blocks -= 1;
      while ( TRUE ) {
        the_thread = _Thread_queue_First( &the_region->Wait_queue );
        if ( the_thread == NULL )
           break;

        the_segment = _Region_Allocate_segment( 
                        the_region, the_thread->Wait.Extra.segment_size );
         
        if ( the_segment == NULL ) 
           break;

        *(void **)the_thread->Wait.return_argument = the_segment;
        the_region->number_of_used_blocks += 1;
        _Thread_queue_Extract( &the_region->Wait_queue, the_thread );
        the_thread->Wait.return_code = STATUS_SUCCESSFUL;
      }

      _Thread_Enable_dispatch();
      return( STATUS_SUCCESSFUL );
  }

  return( STATUS_SUCCESSFUL );   /* unreached - only to remove warnings */
}
