/*  region.h
 * 
 *  This include file contains all the constants and structures associated
 *  with the Region Manager.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __REGION_h
#define __REGION_h

#include "object.h"
#include "threadq.h"
#include "heap.h"

/*
 *  The following records define the control block used to manage
 *  each region.
 */

typedef struct {
  Objects_Control       Object;
  Thread_queue_Control  Wait_queue;            /* waiting threads        */
  void                 *starting_address;      /* physical start addr    */
  unsigned32            length;                /* physical length(bytes) */
  unsigned32            page_size;             /* in bytes               */
  unsigned32            maximum_segment_size;  /* in bytes               */
  Attributes_Control    attribute_set;
  unsigned32            number_of_used_blocks; /* blocks allocated       */
  Heap_Control          Memory;
}  Region_Control;

/*
 *  The following defines the information control block used to
 *  manage this class of objects.
 */

EXTERN Objects_Information _Region_Information;

/*
 *  _Region_Manager_initialization
 *
 *  This routine performs the initialization necessary for this manager.
 */

void _Region_Manager_initialization( 
  unsigned32 maximum_regions
);

/*
 *  _Region_Create
 *
 *  This routine implements the region_create directive.  The
 *  region will have the name name.  The memory area managed by
 *  the region is of length bytes and starts at starting_address.
 *  The memory area will be divided into as many allocatable units of
 *  page_size bytes as possible.   The attribute_set determines which
 *  thread queue discipline is used by the region.  It returns the
 *  id of the created region in ID.
 */

Status_Codes _Region_Create( 
  Objects_Name        name, 
  void               *starting_address,
  unsigned32          length, 
  unsigned32          page_size, 
  Attributes_Control  attribute_set, 
  Objects_Id         *id
);

/*
 *  _Region_Name_to_id
 *
 *  This routine implements the region_ident directive.
 *  This directive returns the region ID associated with name.
 *  If more than one region is named name, then the region
 *  to which the ID belongs is arbitrary.
 */

Status_Codes _Region_Name_to_id( 
  Objects_Name  name, 
  Objects_Id   *id
);

/*
 *  _Region_Delete
 *
 *  This routine implements the region_delete directive.  The
 *  region indicated by ID is deleted.
 */

Status_Codes _Region_Delete( 
  Objects_Id id
);

/*
 *  _Region_Get_segment
 *
 *  This routine implements the region_get_segment directive.  It
 *  attempts to allocate a segment from the region associated with ID.
 *  If a segment of the requested size can be allocated, its address
 *  is returned in segment.  If no segment is available, then the task
 *  may return immediately or block waiting for a segment with an optional
 *  timeout of timeout clock ticks.  Whether the task blocks or returns
 *  immediately is based on the no_wait option in the option_set.
 */

Status_Codes _Region_Get_segment( 
  Objects_Id         id,
  unsigned32         size, 
  Options_Control    option_set, 
  Watchdog_Interval  timeout, 
  void              **segment 
);

/*
 *  _Region_Return_segment
 *
 *  This routine implements the region_return_segment directive.  It
 *  frees the segment to the region associated with ID.  The segment must
 *  have been previously allocated from the same region.  If freeing the
 *  segment results in enough memory being available to satisfy the
 *  region_get_segment of the first blocked task, then that task and as
 *  many subsequent tasks as possible will be unblocked with their requests
 *  satisfied.
 */

Status_Codes _Region_Return_segment( 
  Objects_Id  id,
  void       *segment 
);

/*
 *  _Region_Allocate
 *
 *  This function allocates a region control block from
 *  the inactive chain of free region control blocks.
 */

STATIC INLINE Region_Control *_Region_Allocate( void );

/*
 *  _Region_Free
 *
 *  This routine frees a region control block to the
 *  inactive chain of free region control blocks.
 */

STATIC INLINE void _Region_Free (
  Region_Control *the_region
);

/*
 *  _Region_Get
 *
 *  This function maps region IDs to region control blocks.
 *  If ID corresponds to a local region, then it returns
 *  the_region control pointer which maps to ID and location
 *  is set to OBJECTS_LOCAL.  Otherwise, location is set
 *  to OBJECTS_ERROR and the_region is undefined.
 */

STATIC INLINE Region_Control *_Region_Get (
  Objects_Id         id,
  Objects_Locations *location
);

/*
 *  _Region_Allocate_segment
 *
 *  This function attempts to allocate a segment from the_region.
 *  If successful, it returns the address of the allocated segment.
 *  Otherwise, it returns NULL.
 */

STATIC INLINE void *_Region_Allocate_segment (
  Region_Control *the_region,
  unsigned32      size
);

/*
 *  _Region_Free_segment
 *
 *  This function frees the_segment to the_region.
 */

STATIC INLINE boolean _Region_Free_segment (
  Region_Control *the_region,
  void           *the_segment
);

/*
 *  _Region_Is_null
 *
 *  This function returns TRUE if the_region is NULL and FALSE otherwise.
 */

STATIC INLINE boolean _Region_Is_null (
  Region_Control *the_region
);

#include "region.inl"
#include "regionmp.h"

#endif
/* end of include file */
