/*
 *  Semaphore Manager
 *
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"
#include "attr.h"
#include "config.h"
#include "isr.h"
#include "object.h"
#include "options.h"
#include "sem.h"
#include "states.h"
#include "thread.h"
#include "threadq.h"

/*PAGE
 *
 *  _Semaphore_Manager_initialization
 * 
 *  This routine initializes all semaphore manager related data structures.
 *
 *  Input parameters:
 *    maximum_semaphores - maximum configured semaphores
 * 
 *  Output parameters:  NONE
 */

void _Semaphore_Manager_initialization( 
  unsigned32 maximum_semaphores
)
{
  _Objects_Initialize_information( 
    &_Semaphore_Information, 
    TRUE,
    maximum_semaphores, 
    sizeof( Semaphore_Control )
  );
}

/*PAGE
 *
 *  _Semaphore_Create
 *
 *  This directive creates a semaphore and sets the initial value based
 *  on the given count.  A semaphore id is returned.
 *
 *  Input parameters:
 *    name          - user defined semaphore name
 *    count         - initial count of semaphore
 *    attribute_set - semaphore attributes
 *    id            - pointer to semaphore id
 *
 *  Output parameters:
 *    id       - semaphore id
 *    STATUS_SUCCESSFUL - if successful
 *    error code - if unsuccessful
 */

#include "system.h"
#include "attr.h"
#include "config.h"
#include "object.h"
#include "thread.h"
#include "sem.h"
#include "states.h"

Status_Codes _Semaphore_Create( 
  Objects_Name        name,
  unsigned32          count,
  Attributes_Control  attribute_set,
  Objects_Id         *id
)
{
  register Semaphore_Control *the_semaphore;

  if ( !_Objects_Is_name_valid( name ) ) 
    return ( STATUS_INVALID_NAME );

  if ( _Attributes_Is_global( attribute_set ) ) {
    if ( !_Configuration_Is_multiprocessing() ) 
      return( STATUS_MP_NOT_CONFIGURED );
    if ( _Attributes_Is_inherit_priority( attribute_set ) )
      return( STATUS_NOT_DEFINED );
  }
  else if ( _Attributes_Is_inherit_priority( attribute_set ) )
    if ( ! ( _Attributes_Is_binary_semaphore( attribute_set ) &&
             _Attributes_Is_priority( attribute_set ) ) )
      return( STATUS_NOT_DEFINED );

  if ( _Attributes_Is_binary_semaphore( attribute_set ) && ( count > 1 ) ) 
    return( STATUS_INVALID_NUMBER );

  _Thread_Disable_dispatch();             /* prevents deletion */
  if ( !(the_semaphore = _Semaphore_Allocate()) ) {
    _Thread_Enable_dispatch();
    return( STATUS_TOO_MANY );
  }

  if ( _Attributes_Is_global( attribute_set ) && 
       !( _Objects_MP_Open( &_Semaphore_Information, name,
                            the_semaphore->Object.id, FALSE ) ) ) {
    _Semaphore_Free( the_semaphore );
    _Thread_Enable_dispatch();
    return( STATUS_TOO_MANY );
  }

  the_semaphore->attribute_set = attribute_set;
  the_semaphore->count         = count;
  if ( _Attributes_Is_binary_semaphore( attribute_set ) && count == 0 ) {
    the_semaphore->nest_count = 1;
    the_semaphore->holder     = _Thread_Executing;
    _Thread_Executing->resource_count++; 
  }
  else {
    the_semaphore->nest_count = 0;
    the_semaphore->holder     = NULL;
  }

  _Thread_queue_Initialize( &the_semaphore->Wait_queue, 
                            attribute_set, STATES_WAITING_FOR_SEMAPHORE );
  _Objects_Open( &_Semaphore_Information, &the_semaphore->Object, name );
  *id = the_semaphore->Object.id;
  if ( _Attributes_Is_global( attribute_set ) )
    _Semaphore_MP_Send_process_packet(
      SEMAPHORE_MP_ANNOUNCE_CREATE,
      the_semaphore->Object.id, 
      name,
      0                          /* Not used */
    );
  _Thread_Enable_dispatch();
  return( STATUS_SUCCESSFUL );
}

/*PAGE
 *
 *  _Semaphore_Name_to_id
 *
 *  This directive returns the system ID associated with
 *  the semaphore name.
 *
 *  Input parameters:
 *    name - user defined semaphore name
 *    node - node(s) to be searched
 *    id   - pointer to semaphore id
 *
 *  Output parameters:
 *    *id      - semaphore id
 *    STATUS_SUCCESSFUL - if successful
 *    error code - if unsuccessful
 */

#include "system.h"
#include "object.h"
#include "sem.h"

Status_Codes _Semaphore_Name_to_id( 
  Objects_Name  name,
  unsigned32    node,
  Objects_Id   *id
)
{
  return( _Objects_Name_to_id( &_Semaphore_Information, name, node, id ) );
}

/*PAGE
 *
 *  _Semaphore_Delete
 *
 *  This directive allows a thread to delete a semaphore specified by
 *  the semaphore id.  The semaphore is freed back to the inactive
 *  semaphore chain.
 *
 *  Input parameters:
 *    id - semaphore id
 *
 *  Output parameters:
 *    STATUS_SUCCESSFUL - if successful
 *    error code        - if unsuccessful
 */

#include "system.h"
#include "object.h"
#include "sem.h"
#include "thread.h"

Status_Codes _Semaphore_Delete( 
  Objects_Id id
)
{
  register Semaphore_Control *the_semaphore;
  Objects_Locations           location;

  the_semaphore = _Semaphore_Get( id, &location );
  switch ( location ) {
    case OBJECTS_ERROR:
      return( STATUS_INVALID_ID );
    case OBJECTS_REMOTE:
      _Thread_Dispatch();
      return( STATUS_ILLEGAL_ON_REMOTE_OBJECT );
    case OBJECTS_LOCAL:
      if ( _Attributes_Is_binary_semaphore( the_semaphore->attribute_set) &&
                       ( the_semaphore->count == 0 ) ) {
        _Thread_Enable_dispatch();
        return( STATUS_RESOURCE_IN_USE );
      }

      _Objects_Close( &_Semaphore_Information, &the_semaphore->Object );
      _Thread_queue_Flush( 
        &the_semaphore->Wait_queue,
        _Semaphore_MP_Send_object_was_deleted
      );
      _Semaphore_Free( the_semaphore );
      if ( _Attributes_Is_global( the_semaphore->attribute_set ) )
        _Semaphore_MP_Send_process_packet(
          SEMAPHORE_MP_ANNOUNCE_DELETE,
          the_semaphore->Object.id, 
          0,                         /* Not used */
          0                          /* Not used */
        );
      _Thread_Enable_dispatch();
      return( STATUS_SUCCESSFUL );
  }

  return( STATUS_SUCCESSFUL );   /* unreached - only to remove warnings */
}

/*PAGE
 *
 *  _Semaphore_Obtain
 *
 *  This directive allows a thread to acquire a semaphore.
 *
 *  Input parameters:
 *    id         - semaphore id
 *    option_set - wait option
 *    timeout    - number of ticks to wait (0 means wait forever)
 *
 *  Output parameters:
 *    STATUS_SUCCESSFUL - if successful
 *    error code        - if unsuccessful
 */

#include "system.h"
#include "object.h"
#include "sem.h"
#include "thread.h"
#include "threadq.h"

Status_Codes _Semaphore_Obtain( 
  Objects_Id        id,
  unsigned32        option_set,
  Watchdog_Interval timeout
)
{
  register Semaphore_Control *the_semaphore;
  Objects_Locations           location;

  the_semaphore = _Semaphore_Get( id, &location );
  switch ( location ) {
    case OBJECTS_ERROR:
      return( STATUS_INVALID_ID );
    case OBJECTS_REMOTE:
      return(
        _Semaphore_MP_Send_request_packet(
          SEMAPHORE_MP_OBTAIN_REQUEST,
          id, 
          option_set,
          timeout
        )
      );
    case OBJECTS_LOCAL:
      if ( !_Semaphore_Seize( the_semaphore, option_set ) ) {
        if ( _Attributes_Is_inherit_priority( 
               the_semaphore->attribute_set ) &&
            the_semaphore->holder->current_priority > 
              _Thread_Executing->current_priority ) {
            _Thread_Change_priority( the_semaphore->holder, 
              _Thread_Executing->current_priority );
         }
        _Thread_queue_Enqueue( &the_semaphore->Wait_queue, timeout );
      }
      _Thread_Enable_dispatch();
      return( _Thread_Executing->Wait.return_code );
  }

  return( STATUS_SUCCESSFUL );   /* unreached - only to remove warnings */
}

/*PAGE
 *
 *  _Semaphore_Release
 *
 *  This directive allows a thread to release a semaphore.
 *
 *  Input parameters:
 *    id - semaphore id
 *
 *  Output parameters:
 *    STATUS_SUCCESSFUL - if successful
 *    error code        - if unsuccessful
 */

#include "system.h"
#include "object.h"
#include "sem.h"
#include "thread.h"

Status_Codes _Semaphore_Release( 
  Objects_Id id
)
{
  register Semaphore_Control *the_semaphore;
  Objects_Locations                  location;
  Thread_Control             *the_thread;

  the_semaphore = _Semaphore_Get( id, &location );
  switch ( location ) {
    case OBJECTS_ERROR:
      return( STATUS_INVALID_ID );
    case OBJECTS_REMOTE:
      return(
        _Semaphore_MP_Send_request_packet(
          SEMAPHORE_MP_RELEASE_REQUEST,
          id, 
          0,                               /* Not used */
          0                                /* Not used */
        )
      );
    case OBJECTS_LOCAL:
      if ( _Attributes_Is_binary_semaphore( the_semaphore->attribute_set)) {
        if ( !_Thread_Is_executing( the_semaphore->holder ) ) {
          _Thread_Enable_dispatch();
          return( STATUS_NOT_OWNER_OF_RESOURCE );
        }
   
        the_semaphore->nest_count--;

        if ( the_semaphore->nest_count != 0 ) {
          _Thread_Enable_dispatch();
          return( STATUS_SUCCESSFUL );
        }

        _Thread_Executing->resource_count--;
        the_semaphore->holder = NULL;

        if ( _Attributes_Is_inherit_priority(the_semaphore->attribute_set)&&
             _Thread_Executing->resource_count == 0 &&
             _Thread_Executing->real_priority != 
                _Thread_Executing->current_priority ) {
           _Thread_Change_priority( 
              _Thread_Executing, _Thread_Executing->real_priority );
        }

      }

      if ( (the_thread = 
                _Thread_queue_Dequeue( &the_semaphore->Wait_queue )) ) {

        if ( !_Objects_Is_local_id( the_thread->Object.id ) ) {
          the_thread->receive_packet->return_code = STATUS_SUCCESSFUL;

          _Semaphore_MP_Send_response_packet(
            SEMAPHORE_MP_OBTAIN_RESPONSE,
            id, 
            the_thread  
          );
        }
        else if ( _Attributes_Is_inherit_priority(
                       the_semaphore->attribute_set ) ) {
          _Thread_Executing->resource_count++;
          the_semaphore->holder     = _Thread_Executing;
          the_semaphore->nest_count = 1;
        }
      }
      else
        the_semaphore->count += 1;
      _Thread_Enable_dispatch();
      return( STATUS_SUCCESSFUL );
  }

  return( STATUS_SUCCESSFUL );   /* unreached - only to remove warnings */
}

/*PAGE
 *
 *  _Semaphore_Seize
 * 
 *  This routine attempts to allocate a semaphore to the calling thread.
 * 
 *  Input parameters:
 *    the_semaphore - pointer to semaphore control block
 *    option_set    - acquire semaphore options
 * 
 *  Output parameters:
 *    TRUE  - if semaphore allocated
 *    FALSE - if semaphore NOT allocated
 * 
 *  INTERRUPT LATENCY:
 *    available
 *    wait
 */

#include "system.h"
#include "isr.h"
#include "options.h"
#include "sem.h"
#include "thread.h"

boolean _Semaphore_Seize( 
  Semaphore_Control *the_semaphore,
  Options_Control           option_set
)
{
  Thread_Control *executing;
  ISR_Level              level;

  executing = _Thread_Executing;
  executing->Wait.return_code = STATUS_SUCCESSFUL;
  _ISR_Disable( level );
  if ( the_semaphore->count != 0 ) {
    the_semaphore->count -= 1;
    if ( _Attributes_Is_binary_semaphore( the_semaphore->attribute_set ) ) {
      the_semaphore->holder     = executing;
      the_semaphore->nest_count = 1;
      executing->resource_count++;
    }
    _ISR_Enable( level );
    return( TRUE );
  } 

  if ( _Options_Is_no_wait( option_set ) ) {
    _ISR_Enable( level );
    executing->Wait.return_code = STATUS_UNSATISFIED;
    return( TRUE );
  } 

  if ( _Attributes_Is_binary_semaphore( the_semaphore->attribute_set ) ) {
    if ( the_semaphore->holder == executing ) {
      the_semaphore->nest_count++;
      _ISR_Enable( level );
      return( TRUE );
    }
  }

  the_semaphore->Wait_queue.sync = TRUE;
  executing->Wait.queue      = &the_semaphore->Wait_queue;
  executing->Wait.id         = the_semaphore->Object.id;
  executing->Wait.option_set = option_set;
  _ISR_Enable( level );
  return( FALSE );
}
