/*
 *  Multiprocessing Support for the Semaphore Manager
 * 
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"
#include "mpci.h"
#include "mppkt.h"
#include "object.h"
#include "options.h"
#include "sem.h"
#include "thread.h"
#include "watchdog.h"

/*PAGE
 *
 *  _Semaphore_MP_Send_process_packet
 *
 */

void _Semaphore_MP_Send_process_packet (
  Semaphore_MP_Remote_operations  operation,
  Objects_Id                      semaphore_id,
  Objects_Name                    name,
  Objects_Id                      proxy_id
)
{
  Semaphore_MP_Packet *the_packet;
  unsigned32           node;
 
  switch ( operation ) {
 
    case SEMAPHORE_MP_ANNOUNCE_CREATE:
    case SEMAPHORE_MP_ANNOUNCE_DELETE:
    case SEMAPHORE_MP_EXTRACT_PROXY:
 
      the_packet                    = _Semaphore_MP_Get_packet();
      the_packet->Prefix.class      = MP_PACKET_SEMAPHORE;
      the_packet->Prefix.length     = sizeof ( Semaphore_MP_Packet );
      the_packet->Prefix.to_convert = sizeof ( Semaphore_MP_Packet );
      the_packet->operation         = operation;
      the_packet->Prefix.id         = semaphore_id;
      the_packet->name              = name;
      the_packet->proxy_id          = proxy_id;
 
      if ( operation == SEMAPHORE_MP_EXTRACT_PROXY )
         node = _Objects_Get_node( semaphore_id );
      else
         node = MPCI_ALL_NODES;
 
      _MPCI_Send_process_packet( node, &the_packet->Prefix );
      break;
 
    case SEMAPHORE_MP_OBTAIN_REQUEST:
    case SEMAPHORE_MP_OBTAIN_RESPONSE:
    case SEMAPHORE_MP_RELEASE_REQUEST:
    case SEMAPHORE_MP_RELEASE_RESPONSE:
      break;
  }
}

/*PAGE
 *
 *  _Semaphore_MP_Send_request_packet
 *
 */

Status_Codes _Semaphore_MP_Send_request_packet (
  Semaphore_MP_Remote_operations operation,
  Objects_Id                     semaphore_id,
  Options_Control                option_set,
  Watchdog_Interval              timeout
)
{
  Semaphore_MP_Packet *the_packet;
 
  switch ( operation ) {

    case SEMAPHORE_MP_OBTAIN_REQUEST:
    case SEMAPHORE_MP_RELEASE_REQUEST:

      the_packet                    = _Semaphore_MP_Get_packet();
      the_packet->Prefix.class      = MP_PACKET_SEMAPHORE;
      the_packet->Prefix.length     = sizeof ( Semaphore_MP_Packet );
      the_packet->Prefix.to_convert = sizeof ( Semaphore_MP_Packet );
      the_packet->operation         = operation;
      the_packet->Prefix.id         = semaphore_id;
      the_packet->option_set        = option_set;
      the_packet->timeout           = timeout;

      return (
        _MPCI_Send_request_packet(
          _Objects_Get_node( semaphore_id ),
          &the_packet->Prefix,
          STATES_WAITING_FOR_SEMAPHORE,
          timeout
        )
      ); 
      break;

    case SEMAPHORE_MP_ANNOUNCE_CREATE:
    case SEMAPHORE_MP_ANNOUNCE_DELETE:
    case SEMAPHORE_MP_EXTRACT_PROXY:
    case SEMAPHORE_MP_OBTAIN_RESPONSE:
    case SEMAPHORE_MP_RELEASE_RESPONSE:
      break;

  }
  /*
   *  The following line is included to satisfy compilers which
   *  produce warnings when a function does not end with a return.
   */
  return STATUS_SUCCESSFUL;
}

/*PAGE
 *
 *  _Semaphore_MP_Send_response_packet
 *
 */

void _Semaphore_MP_Send_response_packet (
  Semaphore_MP_Remote_operations  operation,
  Objects_Id                      semaphore_id,
  Thread_Control                 *the_thread
)
{
  Semaphore_MP_Packet *the_packet;
 
  switch ( operation ) {
 
    case SEMAPHORE_MP_OBTAIN_RESPONSE:
    case SEMAPHORE_MP_RELEASE_RESPONSE:
 
      the_packet = ( Semaphore_MP_Packet *) the_thread->receive_packet;
 
/*
 *  The packet being returned already contains the class, length, and
 *  to_convert fields, therefore they are not set in this routine.
 */
      the_packet->operation = operation;
      the_packet->Prefix.id = the_packet->Prefix.source_tid;
 
      _MPCI_Send_response_packet(
        _Objects_Get_node( the_packet->Prefix.source_tid ),
        &the_packet->Prefix
      );
      break;
 
    case SEMAPHORE_MP_ANNOUNCE_CREATE:
    case SEMAPHORE_MP_ANNOUNCE_DELETE:
    case SEMAPHORE_MP_EXTRACT_PROXY:
    case SEMAPHORE_MP_OBTAIN_REQUEST:
    case SEMAPHORE_MP_RELEASE_REQUEST:
      break;
 
  }
}

/*PAGE
 *
 *
 *  _Semaphore_MP_Process_packet
 *
 */

void _Semaphore_MP_Process_packet (
  MP_packet_Prefix  *the_packet_prefix
)
{
  Semaphore_MP_Packet *the_packet;
  Thread_Control      *the_thread;
  boolean              ignored;

  the_packet = (Semaphore_MP_Packet *) the_packet_prefix;

  switch ( the_packet->operation ) {

    case SEMAPHORE_MP_ANNOUNCE_CREATE:

      ignored = _Objects_MP_Open(
                  &_Semaphore_Information,
                  the_packet->name,
                  the_packet->Prefix.id,
                  TRUE
                );  

      _MPCI_Return_packet( the_packet_prefix );
      break;

    case SEMAPHORE_MP_ANNOUNCE_DELETE:

      _Objects_MP_Close(
        &_Semaphore_Information,
        the_packet->Prefix.id
      );
 
      _MPCI_Return_packet( the_packet_prefix );
      break;
 
    case SEMAPHORE_MP_EXTRACT_PROXY:
         
      the_thread = _Thread_MP_Find_proxy( the_packet->proxy_id );
 
      if ( ! _Thread_Is_null( the_thread ) )
        _Thread_queue_Extract( the_thread->Wait.queue, the_thread );
 
      _MPCI_Return_packet( the_packet_prefix );
      break;

    case SEMAPHORE_MP_OBTAIN_REQUEST:
 
      the_packet->Prefix.return_code = _Semaphore_Obtain(
        the_packet->Prefix.id,
        the_packet->option_set,
        the_packet->timeout
      );
 
      if ( ! _Status_Is_proxy_blocking( the_packet->Prefix.return_code ) )
        _Semaphore_MP_Send_response_packet(
           SEMAPHORE_MP_OBTAIN_RESPONSE,
           the_packet->Prefix.id,
           _Thread_Executing
        );
      break;

    case SEMAPHORE_MP_OBTAIN_RESPONSE:
    case SEMAPHORE_MP_RELEASE_RESPONSE:

      the_thread = _MPCI_Process_response( the_packet_prefix );

      _MPCI_Return_packet( the_packet_prefix );
      break;
 
    case SEMAPHORE_MP_RELEASE_REQUEST:
 
      the_packet->Prefix.return_code = _Semaphore_Release(
        the_packet->Prefix.id
      );
 
      _Semaphore_MP_Send_response_packet(
        SEMAPHORE_MP_RELEASE_RESPONSE,
        the_packet->Prefix.id,
        _Thread_Executing
      );
      break;
  } 
}

/*PAGE    
 *
 *  _Semaphore_MP_Send_object_was_deleted
 *
 */

void _Semaphore_MP_Send_object_was_deleted (
  Thread_Control *the_proxy
)
{
  the_proxy->receive_packet->return_code = STATUS_OBJECT_WAS_DELETED;

  _Semaphore_MP_Send_response_packet(
    SEMAPHORE_MP_OBTAIN_RESPONSE,
    the_proxy->Wait.id,
    the_proxy
  );  

}

/*PAGE   
 *
 *  _Semaphore_MP_Send_extract_proxy
 *
 */     
          
void _Semaphore_MP_Send_extract_proxy (
  Thread_Control *the_thread
)
{
  _Semaphore_MP_Send_process_packet(
    SEMAPHORE_MP_EXTRACT_PROXY,
    the_thread->Wait.id,
    (Objects_Name) 0,
    the_thread->Object.id
  );  

}

/*PAGE
 *
 *  _Semaphore_MP_Get_packet
 *
 */

Semaphore_MP_Packet *_Semaphore_MP_Get_packet ( void )
{
  return ( (Semaphore_MP_Packet *) _MPCI_Get_packet() );
}

/* end of file */
