/*  
 *  RTEMS Task Manager
 *
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */ 

#include "system.h"
#include "modes.h"
#include "object.h"
#include "stack.h"
#include "states.h"
#include "tasks.h"
#include "thread.h"
#include "threadq.h"
#include "tod.h"
#include "userext.h"
#include "wkspace.h"

/*PAGE
 *
 *  _RTEMS_tasks_Create
 *
 *  This directive creates a thread by allocating and initializing a 
 *  thread control block and a stack.  The newly created thread is
 *  placed in the dormant state.
 *
 *  Input parameters:
 *    name             - user defined thread name
 *    initial_priority - thread priority
 *    stack_size       - stack size in bytes
 *    initial_modes    - initial thread mode
 *    attribute_set    - thread attributes
 *    id               - pointer to thread id
 *
 *  Output parameters:
 *    id                - thread id
 *    STATUS_SUCCESSFUL - if successful
 *    error code        - if unsuccessful
 */

Status_Codes _RTEMS_tasks_Create( 
  Objects_Name        name,
  Priority_Control    initial_priority,
  unsigned32          stack_size,
  Modes_Control       initial_modes,
  Attributes_Control  attribute_set,
  Objects_Id         *id
)
{
  register Thread_Control *the_thread;
  unsigned32               actual_stack_size;
  unsigned32               memory_needed;
  void                    *memory;

  if ( !_Objects_Is_name_valid( name ) ) 
    return ( STATUS_INVALID_NAME );

  if ( !_Stack_Is_enough( stack_size ) )
    return( STATUS_INVALID_SIZE );

  if ( !_Priority_Is_valid( initial_priority ) )
    return( STATUS_INVALID_PRIORITY );

  if ( _Attributes_Is_global( attribute_set ) &&
       !_Configuration_Is_multiprocessing() )
    return( STATUS_MP_NOT_CONFIGURED );

  _Thread_Disable_dispatch();          /* to prevent deletion */
  if ( !(the_thread = _RTEMS_tasks_Allocate()) ) {
    _Thread_Enable_dispatch();
    return( STATUS_TOO_MANY );
  }
 
  _Stack_Adjust_size( stack_size, actual_stack_size );
  memory_needed = actual_stack_size;

  if ( _Attributes_Is_floating_point( attribute_set ) ) 
    memory_needed += CONTEXT_FP_SIZE;
  
  if ( !(memory = _Workspace_Allocate( memory_needed )) ) {
    _RTEMS_tasks_Free( the_thread );
    _Thread_Enable_dispatch();
    return( STATUS_UNSATISFIED );
  } 

  _Stack_Initialize( 
     &the_thread->Start.Initial_stack, memory, actual_stack_size );

  if ( _Attributes_Is_floating_point( attribute_set ) ) 
    the_thread->fp_context = _Context_Fp_start( memory, actual_stack_size );
  else 
    the_thread->fp_context = NULL;

  the_thread->Start.fp_context = the_thread->fp_context;

  if ( _Attributes_Is_global( attribute_set ) && 
       !( _Objects_MP_Open( &_Thread_Information, name,
                            the_thread->Object.id, FALSE ) ) ) {
    _RTEMS_tasks_Free( the_thread );
    (void) _Workspace_Free( memory );
    _Thread_Enable_dispatch();
    return( STATUS_TOO_MANY );
  }

  the_thread->name                   = name;
  the_thread->attribute_set          = attribute_set;
  the_thread->current_state          = STATES_DORMANT;
  the_thread->current_modes          = initial_modes;
  the_thread->pending_events         = EVENT_SETS_NONE_PENDING;
  the_thread->resource_count         = 0;
  the_thread->real_priority          = initial_priority;
  the_thread->Start.initial_priority = initial_priority;
  the_thread->Start.initial_modes    = initial_modes;

  _Thread_Set_priority( the_thread, initial_priority );

  _ASR_Initialize( &the_thread->Signal );

  _Objects_Open( &_Thread_Information, &the_thread->Object, name );

  *id = the_thread->Object.id;

  _User_extensions_Task_create( the_thread );

  if ( _Attributes_Is_global( attribute_set ) ) 
    _RTEMS_tasks_MP_Send_process_packet(
      RTEMS_TASKS_MP_ANNOUNCE_CREATE,
      the_thread->Object.id, 
      name
    );

  _Thread_Enable_dispatch();
  return( STATUS_SUCCESSFUL );
}

/*PAGE
 *
 *  _RTEMS_tasks_Name_to_id
 *
 *  This directive returns the system ID associated with
 *  the thread name.
 *
 *  Input parameters:
 *    name - user defined thread name
 *    node - node(s) to be searched
 *    id   - pointer to thread id
 *
 *  Output parameters:
 *    *id               - thread id
 *    STATUS_SUCCESSFUL - if successful
 *    error code        - if unsuccessful
 */

Status_Codes _RTEMS_tasks_Name_to_id( 
  Objects_Name  name,
  unsigned32    node,
  Objects_Id   *id
)
{
  if ( name != 0 )
    return( _Objects_Name_to_id( &_Thread_Information, name, node, id ) );

  *id = _Thread_Executing->Object.id;
  return( STATUS_SUCCESSFUL );
}

/*PAGE
 *
 *  _RTEMS_tasks_Start
 *
 *  This directive readies the thread identified by the "id"
 *  based on its current priorty, to await execution.  A thread
 *  can be started only from the dormant state.
 *
 *  Input parameters:
 *    id          - thread id 
 *    entry_point - start execution address of thread
 *    argument    - thread argument
 *
 *  Output parameters:
 *    STATUS_SUCCESSFUL - if successful
 *    error code        - if unsuccessful
 */

Status_Codes _RTEMS_tasks_Start( 
  Objects_Id   id,
  Thread_Entry entry_point,
  unsigned32   argument
)
{
  register Thread_Control *the_thread;
  Objects_Locations               location;

  if ( entry_point == NULL ) 
    return( STATUS_INVALID_ADDRESS );

  the_thread = _Thread_Get( id, &location );
  switch ( location ) {
    case OBJECTS_ERROR:
      return( STATUS_INVALID_ID );
    case OBJECTS_REMOTE:
      _Thread_Dispatch();
      return( STATUS_ILLEGAL_ON_REMOTE_OBJECT );
    case OBJECTS_LOCAL:
      if ( _States_Is_dormant( the_thread->current_state ) ) {
        the_thread->Start.entry_point      = entry_point;
        the_thread->Start.initial_argument = argument;

        _Thread_Load_environment( the_thread );

        _Thread_Ready( the_thread );

        _User_extensions_Task_start( the_thread );

        _Thread_Enable_dispatch();
        return( STATUS_SUCCESSFUL );
      } 
      _Thread_Enable_dispatch();
      return( STATUS_INCORRECT_STATE );
  }

  return( STATUS_SUCCESSFUL );   /* unreached - only to remove warnings */
}

/*PAGE
 *
 *  _RTEMS_tasks_Restart
 *
 *  This directive readies the specified thread.  It restores
 *  the thread environment to the original values established
 *  at thread creation and start time.  A thread can be restarted
 *  from any state except the dormant state.
 *
 *  Input parameters:
 *    id       - thread id 
 *    argument - thread argument 
 *
 *  Output parameters:
 *    STATUS_SUCCESSFUL - if successful
 *    error code        - if unsuccessful
 */

Status_Codes _RTEMS_tasks_Restart( 
  Objects_Id id,
  unsigned32 argument
)
{
  register Thread_Control *the_thread;
  Objects_Locations               location;

  the_thread = _Thread_Get( id, &location );
  switch ( location ) {
    case OBJECTS_ERROR:
      return( STATUS_INVALID_ID );
    case OBJECTS_REMOTE:
      _Thread_Dispatch();
      return( STATUS_ILLEGAL_ON_REMOTE_OBJECT );
    case OBJECTS_LOCAL:
      if ( !_States_Is_dormant( the_thread->current_state ) ) {
        _Thread_Set_transient( the_thread );
        _ASR_Initialize( &the_thread->Signal );
        the_thread->pending_events = EVENT_SETS_NONE_PENDING;
        the_thread->resource_count = 0;
        the_thread->current_modes  = the_thread->Start.initial_modes;
        the_thread->Start.initial_argument = argument;

        _RTEMS_tasks_Cancel_wait( the_thread );

        if ( the_thread->current_priority != 
                         the_thread->Start.initial_priority ) {
          the_thread->real_priority = the_thread->Start.initial_priority;
          _Thread_Set_priority( the_thread, 
             the_thread->Start.initial_priority );
        }

        _Thread_Load_environment( the_thread );

        _Thread_Ready( the_thread );

        _User_extensions_Task_restart( the_thread );

        if ( _Thread_Is_executing ( the_thread ) )
          _Thread_Restart_self();

        _Thread_Enable_dispatch();
        return( STATUS_SUCCESSFUL );
      }
      _Thread_Enable_dispatch();
      return( STATUS_INCORRECT_STATE );
  }

  return( STATUS_SUCCESSFUL );   /* unreached - only to remove warnings */
}

/*PAGE
 *
 *  _RTEMS_tasks_Delete
 *
 *  This directive allows a thread to delete itself or the thread
 *  identified in the id field.  The executive halts execution
 *  of the thread and frees the thread control block.
 *
 *  Input parameters:
 *    id - thread id 
 *
 *  Output parameters:
 *    nothing           - if id is the requesting thread (always succeeds)
 *    STATUS_SUCCESSFUL - if successful and id is 
 *                           not the requesting thread
 *    error code        - if unsuccessful
 */

Status_Codes _RTEMS_tasks_Delete( 
  Objects_Id id
)
{
  register Thread_Control *the_thread;
  Objects_Locations        location;

  the_thread = _Thread_Get( id, &location );
  switch ( location ) {
    case OBJECTS_ERROR:
      return( STATUS_INVALID_ID );
    case OBJECTS_REMOTE:
      _Thread_Dispatch();
      return( STATUS_ILLEGAL_ON_REMOTE_OBJECT );
    case OBJECTS_LOCAL:
      _Objects_Close( &_Thread_Information, &the_thread->Object );

      _Thread_Set_state( the_thread, STATES_TRANSIENT );

      if ( _Thread_Is_allocated_fp( the_thread ) )
        _Thread_Deallocate_fp();

      _User_extensions_Task_delete( the_thread );

      _RTEMS_tasks_Cancel_wait( the_thread );

      (void) _Workspace_Free( the_thread->Start.Initial_stack.area );

      _RTEMS_tasks_Free( the_thread );

      if ( _Attributes_Is_global( the_thread->attribute_set ) )
        _RTEMS_tasks_MP_Send_process_packet(
          RTEMS_TASKS_MP_ANNOUNCE_DELETE,
          the_thread->Object.id, 
          0                                /* Not used */
        );

      _Thread_Enable_dispatch();
      return( STATUS_SUCCESSFUL );
  }

  return( STATUS_SUCCESSFUL );   /* unreached - only to remove warnings */
}

/*PAGE
 *
 *  _RTEMS_tasks_Suspend
 *
 *  This directive will place the specified thread in the "suspended"
 *  state.  Note that the suspended state can be in addition to
 *  other waiting states.
 *
 *  Input parameters:
 *    id - thread id 
 *
 *  Output parameters:
 *    STATUS_SUCCESSFUL - if successful
 *    error code        - if unsuccessful
 */

Status_Codes _RTEMS_tasks_Suspend( 
  Objects_Id id
)
{
  register Thread_Control *the_thread;
  Objects_Locations               location;

  the_thread = _Thread_Get( id, &location );
  switch ( location ) {
    case OBJECTS_ERROR:
      return( STATUS_INVALID_ID );
    case OBJECTS_REMOTE:
      return(
        _RTEMS_tasks_MP_Send_request_packet(
          RTEMS_TASKS_MP_SUSPEND_REQUEST,
          id, 
          0,          /* Not used */
          0,          /* Not used */
          0           /* Not used */
        )
      );
    case OBJECTS_LOCAL:
      if ( !_States_Is_suspended( the_thread->current_state ) ) {
        _Thread_Set_state( the_thread, STATES_SUSPENDED );
        _Thread_Enable_dispatch();
        return( STATUS_SUCCESSFUL );
      } 
      _Thread_Enable_dispatch();
      return( STATUS_ALREADY_SUSPENDED );
  }

  return( STATUS_SUCCESSFUL );   /* unreached - only to remove warnings */
}

/*PAGE
 *
 *  _RTEMS_tasks_Resume
 *
 *  This directive will remove the specified thread 
 *  from the suspended state.
 *
 *  Input parameters:
 *    id - thread id 
 *
 *  Output parameters:
 *    STATUS_SUCCESSFUL - if successful
 *    error code        - if unsuccessful
 */

Status_Codes _RTEMS_tasks_Resume( 
  Objects_Id id
)
{
  register Thread_Control *the_thread;
  Objects_Locations        location;

  the_thread = _Thread_Get( id, &location );
  switch ( location ) {
    case OBJECTS_ERROR: 
      return( STATUS_INVALID_ID );
    case OBJECTS_REMOTE:
      return(
        _RTEMS_tasks_MP_Send_request_packet(
          RTEMS_TASKS_MP_RESUME_REQUEST,
          id, 
          0,          /* Not used */
          0,          /* Not used */
          0           /* Not used */
        )
      );
    case OBJECTS_LOCAL:
      if ( _States_Is_suspended( the_thread->current_state ) ) {
        _Thread_Resume( the_thread );
        _Thread_Enable_dispatch();
        return( STATUS_SUCCESSFUL );
      }
      _Thread_Enable_dispatch();
      return( STATUS_INCORRECT_STATE );
  }

  return( STATUS_SUCCESSFUL );   /* unreached - only to remove warnings */
}   

/*PAGE
 *
 *  _RTEMS_tasks_Set_priority
 *
 *  This directive changes the priority of the specified thread.
 *  The specified thread can be any thread in the system including
 *  the requesting thread.
 *
 *  Input parameters:
 *    id           - thread id (0 indicates requesting thread)
 *    new_priority - thread priority (0 indicates current priority)
 *    old_priority - pointer to previous priority
 *
 *  Output parameters:
 *    old_priority      - previous priority
 *    STATUS_SUCCESSFUL - if successful
 *    error code        - if unsuccessful
 */

Status_Codes _RTEMS_tasks_Set_priority( 
  Objects_Id        id,
  Priority_Control  new_priority,
  Priority_Control *old_priority
)
{
  register Thread_Control *the_thread;
  Objects_Locations               location;

  if ( new_priority != 0 && !_Priority_Is_valid( new_priority ) )
    return( STATUS_INVALID_PRIORITY );

  the_thread = _Thread_Get( id, &location );
  switch ( location ) {
    case OBJECTS_ERROR:
      return( STATUS_INVALID_ID );
    case OBJECTS_REMOTE:
      _Thread_Executing->Wait.return_argument = old_priority;
      return(
        _RTEMS_tasks_MP_Send_request_packet(
          RTEMS_TASKS_MP_SET_PRIORITY_REQUEST,
          id, 
          new_priority,
          0,          /* Not used */
          0           /* Not used */
        )
      );
    case OBJECTS_LOCAL:
      *old_priority = the_thread->current_priority;
      if ( new_priority != 0 ) {
        the_thread->real_priority = new_priority;
        if ( the_thread->resource_count == 0 ||
             the_thread->current_priority > new_priority )
          _Thread_Change_priority( the_thread, new_priority );
      }
      _Thread_Enable_dispatch();
      return( STATUS_SUCCESSFUL );
  }

  return( STATUS_SUCCESSFUL );   /* unreached - only to remove warnings */
}

/*PAGE
 *
 *  _RTEMS_tasks_Mode
 *
 *  This directive enables and disables several modes of
 *  execution for the requesting thread.
 *
 *  Input parameters:
 *    mode_set          - new mode
 *    mask              - mask
 *    previous_mode_set - address of previous mode set
 *
 *  Output:
 *    *previous_mode_set - previous mode set
 *     always returns STATUS_SUCCESSFUL
 */

Status_Codes _RTEMS_tasks_Mode( 
  Modes_Control  mode_set,
  Modes_Control  mask,
  Modes_Control *previous_mode_set
)
{
  if ( _Thread_Change_mode( mode_set, mask, previous_mode_set ) )
    _Thread_Dispatch();
  return( STATUS_SUCCESSFUL );
}

/*PAGE
 *
 *  _RTEMS_tasks_Get_note
 *
 *  This directive obtains the note from the specified notepad
 *  of the specified thread.
 *
 *  Input parameters:
 *    id      - thread id
 *    notepad - notepad number
 *    note    - pointer to note
 *
 *  Output parameters:
 *    note              - filled in if successful
 *    STATUS_SUCCESSFUL - if successful
 *    error code        - if unsuccessful
 */

Status_Codes _RTEMS_tasks_Get_note(
  Objects_Id  id,
  unsigned32  notepad,
  unsigned32 *note
)
{
  register Thread_Control *the_thread;
  Objects_Locations        location;

  if ( notepad > NOTEPAD_15 )
    return( STATUS_INVALID_NUMBER );

  the_thread = _Thread_Get( id, &location );
  switch ( location ) {
    case OBJECTS_ERROR:
      return( STATUS_INVALID_ID );
    case OBJECTS_REMOTE:
      _Thread_Executing->Wait.return_argument = note;
      return(
        _RTEMS_tasks_MP_Send_request_packet(
          RTEMS_TASKS_MP_GET_NOTE_REQUEST,
          id, 
          0,          /* Not used */
          notepad,
          0           /* Not used */
        )
      );
    case OBJECTS_LOCAL:
      *note= the_thread->Notepads[ notepad ];
      _Thread_Enable_dispatch();
      return( STATUS_SUCCESSFUL );
  }

  return( STATUS_SUCCESSFUL );   /* unreached - only to remove warnings */
}

/*PAGE
 *
 * _RTEMS_tasks_Set_note
 *
 *  This directive sets the specified notepad contents to the given
 *  note.
 *
 *  Input parameters:
 *    id      - thread id 
 *    notepad - notepad number
 *    note    - note value
 *
 *  Output parameters:
 *    STATUS_SUCCESSFUL - if successful
 *    error code        - if unsuccessful
 */

Status_Codes _RTEMS_tasks_Set_note( 
  Objects_Id id,
  unsigned32 notepad,
  unsigned32 note
)
{
  register Thread_Control *the_thread;
  Objects_Locations        location;

  if ( notepad > NOTEPAD_15 )
    return( STATUS_INVALID_NUMBER );

  the_thread = _Thread_Get( id, &location );
  switch ( location ) {
    case OBJECTS_ERROR:
      return( STATUS_INVALID_ID );
    case OBJECTS_REMOTE:
      return(
        _RTEMS_tasks_MP_Send_request_packet(
          RTEMS_TASKS_MP_SET_NOTE_REQUEST,
          id, 
          0,          /* Not used */
          notepad,
          note
        )
      );

    case OBJECTS_LOCAL:
      the_thread->Notepads[ notepad ] = note;
      _Thread_Enable_dispatch();
      return( STATUS_SUCCESSFUL );
  }

  return( STATUS_SUCCESSFUL );   /* unreached - only to remove warnings */
}

/*PAGE
 *
 *  _RTEMS_tasks_Wake_after
 *
 *  This directive suspends the requesting thread for the given amount
 *  of ticks.
 *
 *  Input parameters:
 *    ticks - number of ticks to wait
 *
 *  Output parameters:
 *    STATUS_SUCCESSFUL - always successful
 */

Status_Codes _RTEMS_tasks_Wake_after( 
  Watchdog_Interval ticks
)
{
  if ( ticks == 0 ) {
    _Thread_Yield_processor();
    _Thread_Dispatch();
  } else {
    _Thread_Disable_dispatch();
      _Thread_Set_state( _Thread_Executing, STATES_DELAYING );
      _Watchdog_Initialize(
        &_Thread_Executing->Timer, 
        _Thread_Delay_ended,
        _Thread_Executing->Object.id,
        NULL
      );
      _Watchdog_Insert_ticks( &_Thread_Executing->Timer, 
                              ticks, WATCHDOG_ACTIVATE_NOW );
    _Thread_Enable_dispatch();
  }
  return( STATUS_SUCCESSFUL );
}

/*PAGE
 *
 *  _RTEMS_tasks_Wake_when
 *
 *  This directive blocks the requesting thread until the given date and
 *  time is reached.
 *
 *  Input parameters:
 *    time_buffer - pointer to the time and date structure 
 *
 *  Output parameters:
 *    STATUS_SUCCESSFUL - if successful
 *    error code        - if unsuccessful
 */

Status_Codes _RTEMS_tasks_Wake_when( 
TOD_Control *time_buffer
)
{
  Watchdog_Interval seconds;
  Status_Codes      local_result;

  if ( !_TOD_Is_set() )
    return( STATUS_NOT_DEFINED );

  time_buffer->ticks = 0;

  local_result = _TOD_Validate( time_buffer );

  if  ( !_Status_Is_successful( local_result ) )
    return( local_result );

  seconds = _TOD_To_seconds( time_buffer );

  if ( seconds <= _TOD_Seconds_since_epoch )
    return( STATUS_INVALID_CLOCK );
  
  _Thread_Disable_dispatch();
    _Thread_Set_state( _Thread_Executing, STATES_WAITING_FOR_TIME );
    _Watchdog_Initialize(
      &_Thread_Executing->Timer, 
      _Thread_Delay_ended,
      _Thread_Executing->Object.id,
      NULL
    );
    _Watchdog_Insert_seconds( &_Thread_Executing->Timer, 
            seconds - _TOD_Seconds_since_epoch, WATCHDOG_ACTIVATE_NOW );
  _Thread_Enable_dispatch();
  return( STATUS_SUCCESSFUL );
}
