/*  tasks.h
 *
 *  This include file contains all constants and structures associated
 *  with RTEMS tasks.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __RTEMS_TASKS_h
#define __RTEMS_TASKS_h

#include "message.h"
#include "object.h"
#include "part.h"
#include "region.h"
#include "sem.h"
#include "states.h"
#include "thread.h"
#include "threadq.h"

/*
 *  _RTEMS_tasks_Create
 *
 *  This routine implements the task_create directive.  The task
 *  will have the name name.  The attribute_set can be used to indicate
 *  that the task will be globally accessible or utilize floating point.
 *  The task's stack will be stack_size bytes.   The task will begin
 *  execution with initial_priority and initial_modes.  It returns the
 *  id of the created task in ID.
 */

Status_Codes _RTEMS_tasks_Create( 
  Objects_Name        name, 
  Priority_Control    initial_priority, 
  unsigned32          stack_size, 
  Modes_Control       initial_modes, 
  Attributes_Control  attribute_set, 
  Objects_Id         *id
);

/*
 *  _RTEMS_tasks_Delete
 *
 *  This routine implements the task_delete directive.  The
 *  task indicated by ID is deleted.
 */

Status_Codes _RTEMS_tasks_Delete( 
  Objects_Id id
);

/*
 *  _RTEMS_tasks_Name_to_id
 *
 *  This routine implements the task_ident directive.
 *  This directive returns the task ID associated with name.
 *  If more than one task is named name, then the task to
 *  which the ID belongs is arbitrary.  node indicates the
 *  extent of the search for the ID of the task named name.
 *  The search can be limited to a particular node or allowed to
 *  encompass all nodes.
 */

Status_Codes _RTEMS_tasks_Name_to_id( 
  Objects_Name  name, 
  unsigned32    node, 
  Objects_Id   *id
);

/*
 *  _RTEMS_tasks_Get_note
 *
 *  This routine implements the task_get_note directive.  The
 *  value of the indicated notepad for the task associated with ID
 *  is returned in note.
 */

Status_Codes _RTEMS_tasks_Get_note( 
  Objects_Id  id,
  unsigned32  notepad, 
  unsigned32 *note
);

/*
 *  _RTEMS_tasks_Set_note
 *
 *  This routine implements the task_set_note directive.  The
 *  value of the indicated notepad for the task associated with ID
 *  is returned in note.
 */

Status_Codes _RTEMS_tasks_Set_note( 
  Objects_Id id,
  unsigned32 notepad, 
  unsigned32 note
);

/*
 *  _RTEMS_tasks_Mode
 *
 *  This routine implements the task_mode directive.  The current
 *  values of the modes indicated by mask of the calling task are changed
 *  to that indicated in mode_set.  The former mode of the task is
 *  returned in mode_set.
 */

Status_Codes _RTEMS_tasks_Mode( 
  Modes_Control  mode_set, 
  Modes_Control  mask, 
  Modes_Control *previous_mode_set
);

/*
 *  _RTEMS_tasks_Restart
 *
 *  This routine implements the task_restart directive.  The
 *  task associated with ID is restarted at its initial entry
 *  point with the new argument.
 */

Status_Codes _RTEMS_tasks_Restart( 
  Objects_Id id,
  unsigned32 arg
);

/*
 *  _RTEMS_tasks_Suspend
 *
 *  This routine implements the task_suspend directive.  The
 *  SUSPENDED state is set for task associated with ID.
 */

Status_Codes _RTEMS_tasks_Suspend( 
  Objects_Id id
);

/*
 *  _RTEMS_tasks_Resume
 *
 *  This routine implements the task_resume Directive.  The
 *  SUSPENDED state is cleared for task associated with ID.
 */

Status_Codes _RTEMS_tasks_Resume( 
  Objects_Id id
);

/*
 *  _RTEMS_tasks_Set_priority
 *
 *  This routine implements the task_set_priority directive.  The
 *  current priority of the task associated with ID is set to
 *  new_priority.  The former priority of that task is returned
 *  in old_priority.
 */

Status_Codes _RTEMS_tasks_Set_priority( 
  Objects_Id        id,
  Priority_Control  new_priority, 
  Priority_Control *old_priority
);

/*
 *  _RTEMS_tasks_Start
 *
 *  This routine implements the task_start directive.  The
 *  starting execution point of the task associated with ID is
 *  set to entry_point with the initial argument.
 */

Status_Codes _RTEMS_tasks_Start( 
  Objects_Id   id,
  Thread_Entry entry_point, 
  unsigned32   argument
);

/*
 *  _RTEMS_tasks_Wake_when
 *
 *  This routine implements the task_wake_when directive.  The
 *  calling task is blocked until the current time of day is
 *  equal to that indicated by time_buffer.
 */

Status_Codes _RTEMS_tasks_Wake_when( 
  TOD_Control *time_buffer
);

/*
 *  _RTEMS_tasks_Wake_after
 *
 *  This routine implements the task_wake_after directive.  The
 *  calling task is blocked until the indicated number of clock
 *  ticks have occurred.
 */

Status_Codes _RTEMS_tasks_Wake_after( 
  Watchdog_Interval ticks
);

/*
 *  _RTEMS_tasks_Allocate
 *
 *  This function allocates a task control block from
 *  the inactive chain of free task control blocks.
 */

STATIC INLINE Thread_Control *_RTEMS_tasks_Allocate( void );

/*
 *  _RTEMS_tasks_Free
 *
 *  This routine frees a task control block to the
 *  inactive chain of free task control blocks.

 */

STATIC INLINE void _RTEMS_tasks_Free (
  Thread_Control *the_task
);

/*
 *  _RTEMS_tasks_Cancel_wait
 *
 *  This routine unblocks the_thread and cancels any timers
 *  which the_thread has active.
 */

STATIC INLINE void _RTEMS_tasks_Cancel_wait(
  Thread_Control *the_thread
);

#include "tasks.inl"
#include "taskmp.h"

#endif
/* end of include file */
