/*  timer.h
 * 
 *  This include file contains all the constants, structures, and
 *  prototypes associated with the Timer Manager.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __TIMER_h
#define __TIMER_h

#include "object.h"
#include "tod.h"
#include "watchdog.h"

/*
 *  The following enumerated type details the classes to which a timer
 *  may belong.
 */

typedef enum {
  TIMER_INTERVAL,
  TIMER_TIME_OF_DAY,
  TIMER_DORMANT
} Timer_Classes;

/*
 *  The following defines the type of a Timer Service Routine.
 */

typedef Watchdog_Service Timer_Service;

/*
 *  The following enumerated type details the classes to which a timer
 *  may belong.
 */

EXTERN Objects_Information  _Timer_Information;

/*
 *  The following records define the control block used to manage
 *  each timer.
 */

typedef struct {
  Objects_Control  Object;
  Watchdog_Control Ticker;
  Timer_Classes    class;
}   Timer_Control;

/*
 *  _Timer_Manager_initialization
 *
 *  This routine performs the initialization necessary for this manager.
 */

void _Timer_Manager_initialization( 
  unsigned32 maximum_timers
);

/*
 *  _Timer_Create
 *
 *  This routine implements the timer_create directive.  The
 *  timer will have the name name.  It returns the id of the
 *  created timer in ID.
 */

Status_Codes _Timer_Create( 
  Objects_Name  name, 
  Objects_Id   *id 
);

/*
 *  _Timer_Name_to_id
 *
 *  This routine implements the timer_ident directive.
 *  This directive returns the timer ID associated with name.
 *  If more than one timer is named name, then the timer
 *  to which the ID belongs is arbitrary.
 */

Status_Codes _Timer_Name_to_id( 
  Objects_Name  name, 
  Objects_Id   *id 
);

/*
 *  _Timer_Cancel
 *
 *  This routine implements the timer_cancel directive.  It is used
 *  to stop the timer associated with ID from firing.
 */

Status_Codes _Timer_Cancel( 
  Objects_Id id
);

/*
 *  _Timer_Delete
 *
 *  This routine implements the timer_delete directive.  The
 *  timer indicated by ID is deleted.
 */

Status_Codes _Timer_Delete( 
  Objects_Id id 
);

/*
 *  _Timer_Fire_after
 *
 *  This routine implements the timer_fire_after directive.  It
 *  initiates the timer associated with ID to fire in ticks clock
 *  ticks.  When the timer fires, the routine will be invoked.
 */

Status_Codes _Timer_Fire_after( 
  Objects_Id         id, 
  Watchdog_Interval  ticks, 
  Timer_Service      routine,
  void              *user_data
);

/*
 *  _Timer_Fire_when
 *
 *  This routine implements the timer_fire_when directive.  It
 *  initiates the timer associated with ID to fire at wall_time
 *  When the timer fires, the routine will be invoked.
 */

Status_Codes _Timer_Fire_when( 
  Objects_Id         id, 
  TOD_Control       *wall_time, 
  Timer_Service      routine,
  void              *user_data
);

/*
 *  _Timer_Reset
 *
 *  This routine implements the timer_reset directive.  It is used
 *  to reinitialize the interval timer associated with ID just as if
 *  timer_fire_after were re-invoked with the same arguments that
 *  were used to initiate this timer.
 */

Status_Codes _Timer_Reset( 
  Objects_Id id
);

/*
 *  _Timer_Allocate
 *
 *  This function allocates a timer control block from
 *  the inactive chain of free timer control blocks.
 */

STATIC INLINE Timer_Control *_Timer_Allocate( void );

/*
 *  _Timer_Free
 *
 *  This routine frees a timer control block to the
 *  inactive chain of free timer control blocks.
 */

STATIC INLINE void _Timer_Free (
  Timer_Control *the_timer
);

/*
 *  _Timer_Get
 *
 *  This function maps timer IDs to timer control blocks.
 *  If ID corresponds to a local timer, then it returns
 *  the timer control pointer which maps to ID and location
 *  is set to OBJECTS_LOCAL.  Otherwise, location is set
 *  to OBJECTS_ERROR and the returned value is undefined.
 */

STATIC INLINE Timer_Control *_Timer_Get (
  Objects_Id         id,
  Objects_Locations *location
);

/*
 *  _Timer_Is_interval_class
 *
 *  This function returns TRUE if the class is that of an INTERVAL
 *  timer, and FALSE otherwise.
 */

STATIC INLINE boolean _Timer_Is_interval_class (
  Timer_Classes class
);

/*
 *  _Timer_Is_time_of_day_class
 *
 *  This function returns TRUE if the class is that of an INTERVAL
 *  timer, and FALSE otherwise.
 */

STATIC INLINE boolean _Timer_Is_timer_of_day_class (
  Timer_Classes class
);

/*
 *  _Timer_Is_dormant_class
 *
 *  This function returns TRUE if the class is that of a DORMANT
 *  timer, and FALSE otherwise.
 */

STATIC INLINE boolean _Timer_Is_dormant_class (
  Timer_Classes class
);

/*
 *  _Timer_Is_null
 *
 *  This function returns TRUE if the_timer is NULL and FALSE otherwise.
 */

STATIC INLINE boolean _Timer_Is_null (
  Timer_Control *the_timer
);

#include "timer.inl"

#endif
/* end of include file */
