/*  rtems.h
 *
 *  This include file contains information about RTEMS executive that 
 *  is required by the application and is CPU independent.  It includes
 *  two (2) CPU dependent files to tailor its data structures for a 
 *  particular processor.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __RTEMS_GENERIC_h
#define __RTEMS_GENERIC_h

#include "rttypes.h"

/* RTEMS dependent type definitions */

typedef void                            rtems_task;
typedef void                            rtems_isr;
typedef void                            rtems_asr;
typedef void                            rtems_device_driver;
typedef void                            rtems_extension;
typedef void                            rtems_mpci_entry;
typedef void                            rtems_timer_service_routine;

typedef rtems_unsigned32                rtems_task_argument;
typedef rtems_unsigned32                rtems_boolean;
typedef rtems_unsigned32                rtems_name;
typedef rtems_unsigned32                rtems_id;
typedef rtems_unsigned32                rtems_interval;
typedef rtems_unsigned32                rtems_attribute;
typedef rtems_unsigned32                rtems_mode;
typedef rtems_unsigned32                rtems_option;
typedef rtems_unsigned32                rtems_task_priority;
typedef rtems_unsigned32                rtems_notepad_index;
typedef rtems_unsigned32                rtems_event_set;
typedef rtems_unsigned32                rtems_signal_set;
typedef rtems_unsigned32                rtems_device_major_number;
typedef rtems_unsigned32                rtems_device_minor_number;
typedef rtems_unsigned32                rtems_vector_number;
typedef enum status_codes               rtems_status_code;
typedef enum mp_packet_classes          rtems_mp_packet_classes;
typedef rtems_unsigned32                rtems_task_state;
typedef struct rtems_thread_control     rtems_tcb;
typedef struct rtems_tod_control        rtems_time_of_day;

typedef struct configuration_table      rtems_configuration_table;
typedef struct configuration_mpci_table rtems_mpci_table;
typedef struct configuration_initialization_tasks_table
                                        rtems_initialization_tasks_table;
typedef struct configuration_driver_address_table
                                        rtems_driver_address_table;
typedef struct configuration_user_extension_table rtems_extensions_table;
typedef struct configuration_multiprocessing_table
                                        rtems_multiprocessing_table;
typedef struct cpu_configuration_table  rtems_cpu_table;
typedef struct mp_packet_prefix         rtems_mp_packet_prefix;

typedef rtems_task ( *rtems_task_entry )(
                      rtems_task_argument
                   );

typedef rtems_device_driver ( *rtems_device_driver_entry )( 
                 rtems_device_major_number,
                 rtems_device_minor_number,
                 void *,
                 rtems_id,
                 rtems_unsigned32 *
             );

typedef rtems_isr ( *rtems_isr_entry )(
                 rtems_vector_number
             );

typedef rtems_asr ( *rtems_asr_entry )(
                 rtems_signal_set
             );

typedef rtems_extension ( *rtems_tasks_create_extension )(
                 rtems_tcb *,
                 rtems_tcb *
             );

typedef rtems_extension ( *rtems_tasks_delete_extension )(
                 rtems_tcb *,
                 rtems_tcb *
             );

typedef rtems_extension ( *rtems_tasks_start_extension )(
                 rtems_tcb *,
                 rtems_tcb *
             );

typedef rtems_extension ( *rtems_tasks_restart_extension )(
                 rtems_tcb *,
                 rtems_tcb *
             );

typedef rtems_extension ( *rtems_tasks_switch_extension )(
                 rtems_tcb *,
                 rtems_tcb *
             );

typedef rtems_extension ( *rtems_tasks_exitted_extension )(
                 rtems_tcb *
             );

typedef rtems_extension ( *rtems_fatal_extension )(
                 rtems_unsigned32
             );

typedef rtems_mpci_entry ( *rtems_mpci_initialization_entry )( 
                 rtems_configuration_table *
             );

typedef rtems_mpci_entry ( *rtems_mpci_get_packet_entry )( 
                 rtems_unsigned8 **
             );

typedef rtems_mpci_entry ( *rtems_mpci_return_packet_entry )( 
                 rtems_unsigned8 *
             );

typedef rtems_mpci_entry ( *rtems_mpci_send_entry )( 
                 rtems_unsigned32,
                 rtems_unsigned8 *,
                 rtems_unsigned32
             );

typedef rtems_mpci_entry ( *rtems_mpci_receive_entry )( 
                 rtems_unsigned8 **
             );

typedef rtems_timer_service_routine ( *rtems_timer_service_routine_entry )(
                 rtems_id,
                 void *
             );

/* RTEMS CPU Dependent Information */

#include "rtemscpu.h"

/* RTEMS directive completion statuses */

enum status_codes {
  SUCCESSFUL,               /*  0 - successful completion               */
  TASK_EXITTED,             /*  1 - returned from a task                */
  MP_NOT_CONFIGURED,        /*  2 - multiprocessing not configured      */
  INVALID_NAME,             /*  3 - invalid object name                 */
  INVALID_ID,               /*  4 - invalid object id                   */
  TOO_MANY,                 /*  5 - too many                            */
  TIMEOUT,                  /*  6 - timed out waiting                   */
  OBJECT_WAS_DELETED,       /*  7 - object was deleted while waiting    */
  INVALID_SIZE,             /*  8 - specified size was invalid          */
  INVALID_ADDRESS,          /*  9 - address specified is invalid        */
  INVALID_NUMBER,           /* 10 - number was invalid                  */
  NOT_DEFINED,              /* 11 - item has not been initialized       */
  RESOURCE_IN_USE,          /* 12 - resources still outstanding         */
  UNSATISFIED,              /* 13 - request not satisfied               */
  INCORRECT_STATE,          /* 14 - task is in wrong state              */
  ALREADY_SUSPENDED,        /* 15 - task already in state               */
  ILLEGAL_ON_SELF,          /* 16 - illegal operation on calling task   */
  ILLEGAL_ON_REMOTE_OBJECT, /* 17 - illegal operation for remote object */
  CALLED_FROM_ISR,          /* 18 - called from ISR                     */
  INVALID_PRIORITY,         /* 19 - invalid task priority               */
  INVALID_TIME_OF_DAY,      /* 20 - invalid date/time                   */
  INVALID_NODE,             /* 21 - invalid node id                     */
  NOT_CONFIGURED,           /* 22 - directive not configured            */
  NOT_OWNER_OF_RESOURCE,    /* 23 - not owner of resource               */
  NOT_IMPLEMENTED           /* 24 - directive not implemented           */
};

/* Task states */

#define READY                  0x0000  /* ready to run              */
#define DORMANT                0x0001  /* created, not started      */
#define SUSPENDED              0x0002  /* wait to be resumed        */
#define TRANSIENT              0x0004  /* task in transition        */
#define DELAYING               0x0008  /* wait for timeout          */
#define WAITING_FOR_SEGMENT    0x0010  /* wait for segment          */
#define WAITING_FOR_MESSAGE    0x0020  /* wait for message          */
#define WAITING_FOR_EVENT      0x0040  /* wait for event            */
#define WAITING_FOR_SEMAPHORE  0x0080  /* wait for semaphore        */
#define WAITING_FOR_TIME       0x0100  /* wait for date and time    */
#define WAITING_FOR_RPC_REPLY  0x0200  /* wait for rpc reply        */
#define WAITING_FOR_PERIOD     0x0400  /* rate monotonic delay      */

/* attribute constants */

#define DEFAULT_ATTRIBUTES  0x00000000  /* all default options      */

#define NO_FLOATING_POINT   0x00000000  /* no floating point unit   */
#define FLOATING_POINT      0x00000001  /* floating point unit      */

#define LOCAL               0x00000000  /* local object             */
#define GLOBAL              0x00000002  /* global object            */

#define FIFO                0x00000000  /* process in FIFO order    */
#define PRIORITY            0x00000004  /* process by priority      */

#define NO_LIMIT            0x00000000  /* do not place a limit     */
#define LIMIT               0x00000008  /* limit queue entries      */

#define COUNTING_SEMAPHORE  0x000000000
#define BINARY_SEMAPHORE    0x000000010

#define NO_INHERIT_PRIORITY 0x000000000
#define INHERIT_PRIORITY    0x000000020

/* options constants */

#define DEFAULT_OPTIONS     0x00000000  /* wait for completion      */

#define WAIT                0x00000000  /* wait for completion      */
#define NO_WAIT             0x00000001  /* do not wait on resource  */

#define EVENT_ALL           0x00000000  /* wait for all event(s)    */
#define EVENT_ANY           0x00000002  /* wait for any event(s)    */

/* mask constants */

#define ALL_MODE_MASKS      0x0000FFFF  /* all default options      */

#define PREEMPT_MASK        0x00000100  /* disable preemption       */
#define TIMESLICE_MASK      0x00000200  /* enable timeslicing       */
#define ASR_MASK            0x00000400  /* disable ASR              */

/* INTERRUPT_MASK is CPU dependent */   /* disable ASR              */

/* mode constants */

#define DEFAULT_MODES       0x00000000  /* all default options      */

#define PREEMPT             0x00000000  /* enable preemption        */
#define NO_PREEMPT          0x00000100  /* disable preemption       */

#define NO_TIMESLICE        0x00000000  /* disable timeslicing      */
#define TIMESLICE           0x00000200  /* enable timeslicing       */

#define ASR                 0x00000000  /* enable ASR               */
#define NO_ASR              0x00000400  /* disable ASR              */

/* INTERRUPT_LEVEL(level) is CPU dependent */ /* interrupt level    */

#ifndef NULL
#define NULL  0
#endif

/*
 *  Boolean constants to use with rtems_boolean
 */

#if !defined( TRUE ) || (TRUE != 1)
#undef TRUE
#define TRUE     (1)
#endif

#if !defined( FALSE ) || (FALSE != 0)
#undef FALSE
#define FALSE     (0)
#endif

/* identification constants */

#define SEARCH_ALL_NODES    0           /* search all nodes         */
#define WHO_AM_I            0           /* calling task             */

/* miscellaneous constants */

#define CURRENT_MODE        0           /* specify current mode     */
#define CURRENT_PRIORITY    0           /* specify current priority */
#define PENDING_EVENTS      0           /* receive pending events   */
#define NO_TIMEOUT          0           /* wait indefinitely        */
#define SELF                0           /* current task             */
#define PERIOD_STATUS       0           /* periodic timer status    */
#define YIELD_PROCESSOR     0           /* yield CPU (tm_wkafter)   */
#define MINIMUM_PRIORITY    1           /* highest task priority    */
#define MAXIMUM_PRIORITY    255         /* lowest task priority     */
/* MINIMUM_STACK_SIZE  is defined on a per CPU basis */

/* definitions for event set */

#define ALL_EVENTS  0xffffffff
#define EVENT_0     0x00000001
#define EVENT_1     0x00000002
#define EVENT_2     0x00000004
#define EVENT_3     0x00000008
#define EVENT_4     0x00000010
#define EVENT_5     0x00000020
#define EVENT_6     0x00000040
#define EVENT_7     0x00000080
#define EVENT_8     0x00000100
#define EVENT_9     0x00000200
#define EVENT_10    0x00000400
#define EVENT_11    0x00000800
#define EVENT_12    0x00001000
#define EVENT_13    0x00002000
#define EVENT_14    0x00004000
#define EVENT_15    0x00008000
#define EVENT_16    0x00010000
#define EVENT_17    0x00020000
#define EVENT_18    0x00040000
#define EVENT_19    0x00080000
#define EVENT_20    0x00100000
#define EVENT_21    0x00200000
#define EVENT_22    0x00400000
#define EVENT_23    0x00800000
#define EVENT_24    0x01000000
#define EVENT_25    0x02000000
#define EVENT_26    0x04000000
#define EVENT_27    0x08000000
#define EVENT_28    0x10000000
#define EVENT_29    0x20000000
#define EVENT_30    0x40000000
#define EVENT_31    0x80000000

/* definitions for signal set */

#define SIGNAL_0    0x00000001
#define SIGNAL_1    0x00000002
#define SIGNAL_2    0x00000004
#define SIGNAL_3    0x00000008
#define SIGNAL_4    0x00000010
#define SIGNAL_5    0x00000020
#define SIGNAL_6    0x00000040
#define SIGNAL_7    0x00000080
#define SIGNAL_8    0x00000100
#define SIGNAL_9    0x00000200
#define SIGNAL_10   0x00000400
#define SIGNAL_11   0x00000800
#define SIGNAL_12   0x00001000
#define SIGNAL_13   0x00002000
#define SIGNAL_14   0x00004000
#define SIGNAL_15   0x00008000
#define SIGNAL_16   0x00010000
#define SIGNAL_17   0x00020000
#define SIGNAL_18   0x00040000
#define SIGNAL_19   0x00080000
#define SIGNAL_20   0x00100000
#define SIGNAL_21   0x00200000
#define SIGNAL_22   0x00400000
#define SIGNAL_23   0x00800000
#define SIGNAL_24   0x01000000
#define SIGNAL_25   0x02000000
#define SIGNAL_26   0x04000000
#define SIGNAL_27   0x08000000
#define SIGNAL_28   0x10000000
#define SIGNAL_29   0x20000000
#define SIGNAL_30   0x40000000
#define SIGNAL_31   0x80000000

/* notepad location definitions */

#define NOTEPAD_0    0
#define NOTEPAD_1    1
#define NOTEPAD_2    2
#define NOTEPAD_3    3
#define NOTEPAD_4    4
#define NOTEPAD_5    5
#define NOTEPAD_6    6
#define NOTEPAD_7    7
#define NOTEPAD_8    8
#define NOTEPAD_9    9
#define NOTEPAD_10  10
#define NOTEPAD_11  11
#define NOTEPAD_12  12
#define NOTEPAD_13  13
#define NOTEPAD_14  14
#define NOTEPAD_15  15
 
/* Multiprocessing constants */

#define MIN_PKTSIZE  64       /* MPCI layer must support  */
                              /* packets >= this size     */

/*  The following constant defines the number of rtems_unsigned32's 
 *  in a pkt which must be converted to native format in a 
 *  heterogeneous system.  In packets longer than 
 *  MIN_HETERO_CONV rtems_unsigned32's, some of the "extra" data may a 
 *  user message buffer which is not automatically endian swapped.
 */

#define MIN_HETERO_CONV  ( sizeof( rtems_mp_packet_prefix ) / 4 )

enum mp_packet_classes {
  MP_PACKET_INTERNAL_THREADS = 0,
  MP_PACKET_RTEMS_TASKS      = 1,
  MP_PACKET_MESSAGE_QUEUE    = 2,
  MP_PACKET_SEMAPHORE        = 3,
  MP_PACKET_PARTITION        = 4,
  MP_PACKET_REGION           = 5,
  MP_PACKET_EVENT            = 6,
  MP_PACKET_SIGNAL           = 7 
};

/*
 *  Macros to access sub-fields in an object ID
 */

#define get_node( id )   (id >> 16)    /* MSW = local node id */
#define get_index( id )  (id & 0xFFFF) /* LSW = object's id   */

/*
 *  Macros to assist in manipulating object names.
 */
 
#define build_name( C1, C2, C3, C4 ) \
   ( C1 << 24 | C2 << 16 | C3 << 8 | C4 )

#define name_to_characters( NAME, C1, C2, C3, C4 ) \
  { \
    (C1) = ((NAME) >> 24) & 0xFF; \
    (C2) = ((NAME) >> 16) & 0xFF; \
    (C3) = ((NAME) >>  8) & 0xFF; \
    (C4) =  (NAME)        & 0xFF; \
  }

/* time of day control block */

struct rtems_tod_control {
  rtems_unsigned32  year;               /* year, A.D.                 */
  rtems_unsigned32  month;              /* month, 1 -> 12             */
  rtems_unsigned32  day;                /* day, 1 -> 31               */
  rtems_unsigned32  hour;               /* hour, 0 -> 23              */
  rtems_unsigned32  minute;             /* minute, 0 -> 59            */
  rtems_unsigned32  second;             /* second, 0 -> 59            */
  rtems_unsigned32  ticks;              /* elapsed ticks between secs */
};

/* task control block */

struct rtems_thread_control {
  rtems_tcb           *next;           /* pointer to next TCB        */
  rtems_tcb           *prev;           /* pointer to previous TCB    */
  rtems_id             tid;            /* task identification        */
  rtems_name           name;           /* task name                  */
  rtems_unsigned32     current_state;  /* task state                 */
  rtems_task_priority  current_priority; /* current task priority    */
  rtems_task_priority  real_priority;  /* non-inherited priority     */
  rtems_unsigned8      reserved[ RTEMS_TCB_RESERVED_AREA ];
                                       /* reserved for RTEMS         */
  rtems_mode           mode;           /* current task mode          */
  rtems_unsigned32     attributes;     /* task attributes            */
  struct registers     Regs;           /* CPU register save area     */
  rtems_unsigned8     *fp_context;     /* pointer to FP context area */
  rtems_unsigned32     notepad[16];    /* executive notepads         */
  rtems_unsigned8     *extension;      /* pointer TCB extension      */
};

/* CPU information */

/* struct cpu_configuration_table is CPU dependent */

/* init task(s) information */

struct configuration_initialization_tasks_table {
  rtems_name           name;              /* task name                */
  rtems_unsigned32     stack_size;        /* task stack size          */
  rtems_task_priority  initial_priority;  /* task priority            */
  rtems_attribute      attribute_set;     /* task attributes          */
  rtems_task_entry     entry_point;       /* task entry point         */
  rtems_mode           mode_set;          /* task initial mode        */
  rtems_task_argument  argument;          /* task argument            */
};

/* device driver table */

struct configuration_driver_address_table {
  rtems_device_driver_entry  initialization;  /* I/O init routine     */
  rtems_device_driver_entry  open;            /* I/O open routine     */
  rtems_device_driver_entry  close;           /* I/O close routine    */
  rtems_device_driver_entry  read;            /* I/O read routine     */
  rtems_device_driver_entry  write;           /* I/O write routine    */
  rtems_device_driver_entry  control;         /* I/O control routine  */
};

/* user extensions table */

struct configuration_user_extension_table {
  rtems_tasks_create_extension   task_create;   /* task create extension  */
  rtems_tasks_start_extension    task_start;    /* task start extension   */
  rtems_tasks_restart_extension  task_restart;  /* task restart extension */
  rtems_tasks_delete_extension   task_delete;   /* task delete extension  */
  rtems_tasks_switch_extension   task_switch;   /* task switch extension  */
  rtems_tasks_exitted_extension  task_exitted;  /* task exitted extension */
  rtems_fatal_extension          fatal;         /* fatal error extension  */
};

/* multiprocessor communications interface table */

struct configuration_mpci_table {
  rtems_mpci_initialization_entry  initialization;  /* mpci init routine  */
  rtems_mpci_get_packet_entry      get_packet;      /* mpci get packet    */
  rtems_mpci_return_packet_entry   return_packet;   /* mpci return packet */
  rtems_mpci_send_entry            send_packet;     /* mpci send routine  */
  rtems_mpci_receive_entry         receive_packet;  /* mpci receive       */
};

/* multiprocessor configuration table */

struct configuration_multiprocessing_table {
  rtems_unsigned32   node;                    /* local node number        */
  rtems_unsigned32   maximum_nodes;           /* maximum # nodes          */
  rtems_unsigned32   maximum_global_objects;  /* maximum # global objects */
  rtems_unsigned32   maximum_proxies;         /* maximum # proxies        */
  rtems_mpci_table  *User_mpci_table;         /* pointer to MPCI table    */
};

/* configuration table */

struct configuration_table {
  void                              *work_space_start;
  rtems_unsigned32                   work_space_size;
  rtems_unsigned32                   maximum_tasks;
  rtems_unsigned32                   maximum_timers;
  rtems_unsigned32                   maximum_semaphores;
  rtems_unsigned32                   maximum_message_queues;
  rtems_unsigned32                   maximum_messages;
  rtems_unsigned32                   maximum_partitions;
  rtems_unsigned32                   maximum_regions;
  rtems_unsigned32                   maximum_ports;
  rtems_unsigned32                   maximum_periods;
  rtems_unsigned32                   milliseconds_per_tick;
  rtems_unsigned32                   ticks_per_timeslice;
  rtems_unsigned32                   number_of_initialization_tasks;
  rtems_initialization_tasks_table  *User_initialization_tasks_table;
  rtems_unsigned32                   number_of_device_drivers;
  rtems_driver_address_table        *Device_driver_table;
  rtems_extensions_table            *User_extension_table;
  rtems_multiprocessing_table       *User_multiprocessing_table;
};

/* multiprocessor packet prefix */

struct mp_packet_prefix {
  rtems_mp_packet_classes class;
  rtems_id                id;
  rtems_id                source_tid;
  rtems_task_priority     source_priority;
  rtems_status_code       return_code;
  rtems_unsigned32        length;
  rtems_unsigned32        to_convert;
  rtems_unsigned32        pad0; /* makes this multiple of 16 bytes */
};

/* C Interface Definitions */

/*
 *  The following macros are used to map user directive
 *  names to the C Interface entry points.  The exact 
 *  names of the C Interface entry points may vary 
 *  between compilers.
 */

/* Initialization Manager Directives */

#define     initialize_executive( configuration_table, cpu_table ) \
           _Initialize_Executive( configuration_table, cpu_table )
#define     shutdown_executive( result ) \
           _Shutdown_Executive( result )

/* Tasks Manager Directives */

#define     task_create( name, initial_priority, stack_size, \
                         initial_mode, attribute_set, id ) \
           _RTEMS_tasks_Create( name, initial_priority, stack_size, \
                          initial_mode, attribute_set, id )
#define     task_ident( name, node, id ) \
           _RTEMS_tasks_Name_to_id( name, node, id )
#define     task_start( id, entry_point, argument ) \
           _RTEMS_tasks_Start( id, entry_point, argument )
#define     task_restart( id, argument ) \
           _RTEMS_tasks_Restart( id, argument )
#define     task_delete( id ) \
           _RTEMS_tasks_Delete( id )
#define     task_suspend( id ) \
           _RTEMS_tasks_Suspend( id )
#define     task_resume( id ) \
           _RTEMS_tasks_Resume( id )
#define     task_set_priority( id, new_priority, old_priority ) \
           _RTEMS_tasks_Set_priority( id, new_priority, old_priority )
#define     task_mode( mode_set, mask, previous_mode_set ) \
           _RTEMS_tasks_Mode( mode_set, mask, previous_mode_set )
#define     task_get_note( id, notepad, note ) \
           _RTEMS_tasks_Get_note( id, notepad, note )
#define     task_set_note( id, notepad, note ) \
           _RTEMS_tasks_Set_note( id, notepad, note )
#define     task_wake_after( ticks ) \
           _RTEMS_tasks_Wake_after( ticks )
#define     task_wake_when( time_buffer ) \
           _RTEMS_tasks_Wake_when( time_buffer )

/* Interrupt Manager Directives */

#define     interrupt_catch( new_isr_handler, vector, old_isr_handler ) \
           _Interrupt_Catch( new_isr_handler, vector, old_isr_handler ) 

/* Clock Manager Directives */

#define     clock_get( time_buffer ) \
           _Clock_Get( time_buffer )
#define     clock_set( time_buffer ) \
           _Clock_Set( time_buffer )
#define     clock_tick() \
           _Clock_Tick()

/* Timer Manager Directives */

#define     timer_create( name, id ) \
           _Timer_Create( name, id )
#define     timer_ident( name, id ) \
           _Timer_Name_to_id( name, id )
#define     timer_cancel( id ) \
           _Timer_Cancel( id )
#define     timer_delete( id ) \
           _Timer_Delete( id )
#define     timer_fire_after( id, ticks, routine, user_data ) \
           _Timer_Fire_after( id, ticks, routine, user_data )
#define     timer_fire_when( id, wall_time, routine, user_data ) \
           _Timer_Fire_when( id, wall_time, routine, user_data )
#define     timer_reset( id ) \
           _Timer_Reset( id )

/* Semaphore Manager Directives */

#define     semaphore_create( name, count, attribute_set, id ) \
           _Semaphore_Create( name, count, attribute_set, id )
#define     semaphore_ident( name, node, id ) \
           _Semaphore_Name_to_id( name, node, id )
#define     semaphore_delete( id ) \
           _Semaphore_Delete( id )
#define     semaphore_obtain( id, option_set, timeout ) \
           _Semaphore_Obtain( id, option_set, timeout )
#define     semaphore_release( id ) \
           _Semaphore_Release( id )

/* Message Queue Manager Directives */

#define     message_queue_create( name, count, attribute_set, id ) \
           _Message_queue_Create( name, count, attribute_set, id )
#define     message_queue_ident( name, node, id ) \
           _Message_queue_Name_to_id( name, node, id )
#define     message_queue_delete( id ) \
           _Message_queue_Delete( id )
#define     message_queue_send( id, buffer ) \
           _Message_queue_Send( id, buffer )
#define     message_queue_urgent( id, buffer ) \
           _Message_queue_Urgent( id, buffer )
#define     message_queue_broadcast( id, buffer, count ) \
           _Message_queue_Broadcast( id, buffer, count )
#define     message_queue_receive( id, buffer, option_set, timeout ) \
           _Message_queue_Receive( id, buffer, option_set, timeout )
#define     message_queue_flush( id, count ) \
           _Message_queue_Flush( id, count )

/* Event Manager Directives */

#define     event_send( id, event_in ) \
           _Event_Send( id, event_in )
#define     event_receive( event_in, option_set, ticks, event_out ) \
           _Event_Receive( event_in, option_set, ticks, event_out )

/* Signal Manager Directives */

#define     signal_catch( asr_handler, mode_set ) \
           _Signal_Catch( asr_handler, mode_set ) 
#define     signal_send( id, signal_set ) \
           _Signal_Send( id, signal_set )

/* Partition Manager Directives */

#define     partition_create( name, starting_address, length, \
                              buffer_size, attribute_set, id ) \
           _Partition_Create( name, starting_address, length, \
                              buffer_size, attribute_set, id )
#define     partition_ident( name, node, id ) \
           _Partition_Name_to_id( name, node, id )
#define     partition_delete( id ) \
           _Partition_Delete( id )
#define     partition_get_buffer( id, buffer ) \
           _Partition_Get_buffer( id, (void **)buffer )
#define     partition_return_buffer( id, buffer ) \
           _Partition_Return_buffer( id, (void *)buffer )

/* Region Manager Directives */

#define     region_create( name, starting_address, length, page_size, \
                           attribute_set, id ) \
           _Region_Create( name, starting_address, length, page_size, \
                           attribute_set, id )
#define     region_ident( name, id ) \
           _Region_Name_to_id( name, id )
#define     region_delete( id ) \
           _Region_Delete( id )
#define     region_get_segment( id, size, option_set, timeout, segment ) \
           _Region_Get_segment( id, size, option_set, \
                                             timeout, (void **)segment )
#define     region_return_segment( id, segment ) \
           _Region_Return_segment( id, (void *)segment )

/* Dual-Ported Memory Manager Directives */

#define     port_create( name, internal_start, \
                         external_start, length, id ) \
           _Dual_ported_memory_Create( name, internal_start, \
                                       external_start, length, id )
#define     port_ident( name, id ) \
           _Dual_ported_memory_Name_to_id( name, id )
#define     port_delete( id ) \
           _Dual_ported_memory_Delete( id )
#define     port_external_to_internal( id, external, internal ) \
           _Dual_ported_memory_External_to_internal( \
                                id, (void *)external, (void **)internal )
#define     port_internal_to_external( id, internal, external ) \
           _Dual_ported_memory_Internal_to_external( \
                                id, (void *)internal, (void **)external )

/* Input/Output Manager Directives */

#define     io_initialize( major, minor, argument, return_value ) \
           _IO_Initialize( major, minor, argument, return_value )
#define     io_open( major, minor, argument, return_value ) \
           _IO_Open( major, minor, argument, return_value )
#define     io_close( major, minor, argument, return_value ) \
           _IO_Close( major, minor, argument, return_value )
#define     io_read( major, minor, argument, return_value ) \
           _IO_Read( major, minor, argument, return_value )
#define     io_write( major, minor, argument, return_value ) \
           _IO_Write( major, minor, argument, return_value )
#define     io_control( major, minor, argument, return_value ) \
           _IO_Control( major, minor, argument, return_value )

/* Fatal Manager Directives */

#define     fatal_error_occurred( the_error ) \
           _Fatal_Error_occurred( the_error )

/* Rate Monotonic Manager Directives */

#define     rate_monotonic_create( name, id ) \
           _Rate_monotonic_Create( name, id )
#define     rate_monotonic_ident( name, id ) \
           _Rate_monotonic_Name_to_id( name, id )
#define     rate_monotonic_cancel( id ) \
           _Rate_monotonic_Cancel( id )
#define     rate_monotonic_delete( id ) \
           _Rate_monotonic_Delete( id )
#define     rate_monotonic_period( id, length ) \
           _Rate_monotonic_Period( id, length )

/* Multiprocessing Manager Directives */

#define     multiprocessing_announce() \
           _Multiprocessing_Announce() 

/* C Language Directive Prototypes */

/* Initialization Manager Directives */

void _Initialize_Executive(
  rtems_configuration_table *configuration_table,
  rtems_cpu_table           *cpu_table
);

void _Shutdown_Executive(
  rtems_unsigned32 result
);

/* Tasks Manager Directives */

rtems_status_code _RTEMS_tasks_Create(
  rtems_name           name,
  rtems_task_priority  initial_priority,
  rtems_unsigned32     stack_size,
  rtems_mode           initial_modes,
  rtems_attribute      attribute_set,
  rtems_id            *id
);

rtems_status_code _RTEMS_tasks_Delete(
  rtems_id id
);

rtems_status_code _RTEMS_tasks_Get_note(
  rtems_id          id,
  rtems_unsigned32  notepad,
  rtems_unsigned32 *note
);

rtems_status_code _RTEMS_tasks_Mode(
  rtems_mode  mode_set,
  rtems_mode  mask,
  rtems_mode *previous_mode_set
);

rtems_status_code _RTEMS_tasks_Name_to_id(
  rtems_name        name,
  rtems_unsigned32  node,
  rtems_id         *id
);

rtems_status_code _RTEMS_tasks_Restart(
  rtems_id            id,
  rtems_task_argument argument
);

rtems_status_code _RTEMS_tasks_Resume(
  rtems_id id
);

rtems_status_code _RTEMS_tasks_Set_note(
  rtems_id         id,
  rtems_unsigned32 notepad,
  rtems_unsigned32 note
);

rtems_status_code _RTEMS_tasks_Set_priority(
  rtems_id             id,
  rtems_task_priority  new_priority,
  rtems_task_priority *old_priority
);

rtems_status_code _RTEMS_tasks_Start(
  rtems_id            id,
  rtems_task_entry    entry_point,
  rtems_task_argument argument
);

rtems_status_code _RTEMS_tasks_Suspend(
  rtems_id id
);


rtems_status_code _RTEMS_tasks_Wake_after(
  rtems_interval ticks
);

rtems_status_code _RTEMS_tasks_Wake_when(
  rtems_time_of_day *time_buffer
);

/* Interrupt Manager Directives */

rtems_status_code _Interrupt_Catch(
  rtems_isr_entry      new_isr_handler,
  rtems_vector_number  vector,
  rtems_isr_entry     *old_isr_handler
);

/* Clock Manager Directives */

rtems_status_code _Clock_Get(
  rtems_time_of_day *time_buffer
);

rtems_status_code _Clock_Set(
  rtems_time_of_day *time_buffer
);

rtems_status_code _Clock_Tick( void );

/* Timer Manager Directives */

rtems_status_code _Timer_Create(
  rtems_name  name,
  rtems_id   *id
);

rtems_status_code _Timer_Name_to_id(
  rtems_name  name,
  rtems_id   *id
);

rtems_status_code _Timer_Cancel(
  rtems_id id
);

rtems_status_code _Timer_Delete(
  rtems_id id
);

rtems_status_code _Timer_Fire_after(
  rtems_id                           id,
  rtems_interval                     ticks,
  rtems_timer_service_routine_entry  routine,
  void                              *user_data
);

rtems_status_code _Timer_Fire_when(
  rtems_id                           id,
  rtems_time_of_day                 *wall_time,
  rtems_timer_service_routine_entry  routine,
  void                              *user_data
);

rtems_status_code _Timer_Reset(
  rtems_id   id
);

/* Semaphore Manager Directives */

rtems_status_code _Semaphore_Create(
  rtems_name        name,
  rtems_unsigned32  count,
  rtems_attribute   attribute_set,
  rtems_id         *id
);

rtems_status_code _Semaphore_Name_to_id(
  rtems_name        name,
  rtems_unsigned32  node,
  rtems_id         *id
);

rtems_status_code _Semaphore_Delete(
  rtems_id id
);

rtems_status_code _Semaphore_Obtain(
  rtems_id         id,
  rtems_unsigned32 option_set,
  rtems_interval   timeout
);

rtems_status_code _Semaphore_Release(
  rtems_id id
);

/* Message Queue Manager Directives */

rtems_status_code _Message_queue_Create(
  rtems_name        name,
  rtems_unsigned32  count,
  rtems_attribute   attribute_set,
  rtems_id         *id
);

rtems_status_code _Message_queue_Name_to_id(
  rtems_name        name,
  rtems_unsigned32  node,
  rtems_id         *id
);

rtems_status_code _Message_queue_Delete(
  rtems_id id
);

rtems_status_code _Message_queue_Send(
  rtems_id id,
  long     (*buffer)[4]
);

rtems_status_code _Message_queue_Urgent(
  rtems_id id,
  long     (*buffer)[4]
);

rtems_status_code _Message_queue_Broadcast(
  rtems_id           id,
  long             (*buffer)[4],
  rtems_unsigned32  *count
);

rtems_status_code _Message_queue_Receive(
  rtems_id           id,
  long             (*buffer)[4],
  rtems_unsigned32  option_set,
  rtems_interval    timeout
);

rtems_status_code _Message_queue_Flush(
  rtems_id          id,
  rtems_unsigned32 *count
);

/* Event Manager Directives */

rtems_status_code _Event_Send (
  rtems_id         id,
  rtems_event_set  event_in
);

rtems_status_code _Event_Receive (
  rtems_event_set  event_in,
  rtems_option     option_set,
  rtems_interval   ticks,
  rtems_event_set *event_out
);

/* Signal Manager Directives */

rtems_status_code _Signal_Catch(
  rtems_asr_entry  asr_handler,
  rtems_mode mode
);

rtems_status_code _Signal_Send(
  rtems_id         id,
  rtems_signal_set signal_set
);

/* Partition Manager Directives */

rtems_status_code _Partition_Create(
  rtems_name        name,
  void             *starting_address,
  rtems_unsigned32  length,
  rtems_unsigned32  buffer_size,
  rtems_attribute   attribute_set,
  rtems_id         *id
);

rtems_status_code _Partition_Name_to_id(
  rtems_name        name,
  rtems_unsigned32  node,
  rtems_id         *id
);

rtems_status_code _Partition_Delete(
  rtems_id id
);

rtems_status_code _Partition_Get_buffer(
  rtems_id   id,
  void     **buffer
);

rtems_status_code _Partition_Return_buffer(
  rtems_id  id,
  void     *buffer
);

/* Region Manager Directives */

rtems_status_code _Region_Create(
  rtems_name        name,
  void             *starting_address,
  rtems_unsigned32  length,
  rtems_unsigned32  page_size,
  rtems_attribute   attribute_set,
  rtems_id         *id
);

rtems_status_code _Region_Name_to_id(
  rtems_name  name,
  rtems_id   *id
);

rtems_status_code _Region_Delete(
  rtems_id id
);

rtems_status_code _Region_Get_segment(
  rtems_id            id,
  rtems_unsigned32    size,
  rtems_option        option_set,
  rtems_interval      timeout,
  void              **segment
);

rtems_status_code _Region_Return_segment(
  rtems_id  id,
  void     *segment
);

/* Dual-Ported Memory Manager Directives */

rtems_status_code _Dual_ported_memory_Create(
  rtems_name        name,
  void             *internal_start,
  void             *external_start,
  rtems_unsigned32  length,
  rtems_id         *id
);

rtems_status_code _Dual_ported_memory_Name_to_id(
  rtems_name  name,
  rtems_id   *id
);

rtems_status_code _Dual_ported_memory_Delete(
  rtems_id id
);

rtems_status_code _Dual_ported_memory_External_to_internal(
  rtems_id   id,
  void      *external,
  void     **internal
);

rtems_status_code _Dual_ported_memory_Internal_to_external(
  rtems_id   id,
  void      *internal,
  void     **external
);

/* Input/Output Manager Directives */

rtems_status_code _IO_Initialize(
  rtems_device_major_number  major,
  rtems_device_minor_number  minor,
  void                      *argument,
  rtems_unsigned32          *return_value
);

rtems_status_code _IO_Open(
  rtems_device_major_number  major,
  rtems_device_minor_number  minor,
  void                      *argument,
  rtems_unsigned32          *return_value
);

rtems_status_code _IO_Close(
  rtems_device_major_number  major,
  rtems_device_minor_number  minor,
  void                      *argument,
  rtems_unsigned32          *return_value
);

rtems_status_code _IO_Read(
  rtems_device_major_number  major,
  rtems_device_minor_number  minor,
  void                      *argument,
  rtems_unsigned32          *return_value
);

rtems_status_code _IO_Write(
  rtems_device_major_number  major,
  rtems_device_minor_number  minor,
  void                      *argument,
  rtems_unsigned32          *return_value
);

rtems_status_code _IO_Control(
  rtems_device_major_number  major,
  rtems_device_minor_number  minor,
  void                      *argument,
  rtems_unsigned32          *return_value
);

/* Fatal Manager Directives */

void volatile _Fatal_Error_occurred(
  rtems_unsigned32  the_error
);

/* Rate Monotonic Manager Directives */

rtems_status_code _Rate_monotonic_Create(
  rtems_name  name,
  rtems_id   *id
);

rtems_status_code _Rate_monotonic_Name_to_id(
  rtems_name  name,
  rtems_id   *id
);

rtems_status_code _Rate_monotonic_Cancel(
  rtems_id id
);

rtems_status_code _Rate_monotonic_Delete(
  rtems_id id
);

rtems_status_code _Rate_monotonic_Period(
  rtems_id       id,
  rtems_interval length
);

/* Multiprocessing Manager Directives */

void _Multiprocessing_Announce( void );

/*
 *  Support functions
 */

/*
 *  Status code related functions 
 */

#define rtems_is_status_successful( _status ) \
  ( (_status) == SUCCESSFUL )

#define rtems_are_statuses_successful( _status1, _status2 ) \
  ( (_status1) == (_status2) )

#endif
/* end of include file */
