/*  shm.h
 *
 *  This include file contains all the constants, structures, 
 *  and global variables for this RTEMS based shared memory 
 *  communications interface driver.
 *
 *  Processor board dependencies are in other files.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __SHM_h
#define __SHM_h

#include "shmcpu.h"

/*  The information contained in the Node Status, Locked Queue, and 
 *  Envelope Control Blocks must be maintained in a NEUTRAL format.
 *  Currently the neutral format may be selected as big or little 
 *  endian by simply defining either NEUTRAL_BIG or NEUTRAL_LITTLE.  
 *
 *  It is CRITICAL to note that the neutral format can ONLY be 
 *  changed by modifying this file and recompiling the ENTIRE
 *  SHM driver including ALL target specific support files.
 *
 *  The following table details the memory contents for the endian 
 *  field of the Node Status Control Block in the various 
 *  data format configurations (data is in hexadecimal):
 *
 *   NEUTRAL NATIVE BYTE 0  BYTE 1  BYTE 2  BYTE 3
 *   ======= ====== ======  ======  ======  ======
 *    BIG     BIG     00      00      00      01
 *    BIG    LITTLE   10      00      00      00
 *   LITTLE   BIG     01      00      00      00
 *   LITTLE  LITTLE   00      00      00      10
 *
 *  The locking mechanism is based upon the MC68020 "tas" 
 *  instruction which is atomic.  All ports to other CPUs
 *  comply with the restrictive placement of lock bit by this
 *  instruction.  The lock bit is the most significant bit in a 
 *  big-endian rtems_unsigned32.  On other processors, the lock is 
 *  typically implemented via an atomic swap or atomic modify
 *  bits type instruction.
 */

#define NEUTRAL_BIG

#ifdef NEUTRAL_BIG
#define SHM_BIG       0x00000001
#define SHM_LITTLE    0x10000000
#endif

#ifdef NEUTRAL_LITTLE
#define SHM_BIG       0x01000000
#define SHM_LITTLE    0x00000010
#endif

#define START_SHARED_MEM  Shm_config->base
#define SHARED_MEM_LEN    Shm_config->length
#define SHM_ENDIAN        Shm_config->format
#define Int_address(node) Shm_config->Intr.address
#define Int_value(node)   Shm_config->Intr.value
#define Int_length(node)  Shm_config->Intr.length
#define Shm_convert( value ) \
  ((Shm_config->convert) ? (*Shm_config->convert)(value) : (value))

/* constants */

#define MASTER                  1     /* master initialization node */
#define FIRST_NODE              1

/* size constants */

#define KILOBYTE          (1024)
#define MEGABYTE          (1024*1024)

/* inter-node interrupt values */

#define NO_INTERRUPT            0     /* used for polled nodes */
#define BYTE                    1
#define WORD                    2
#define LONG                    4

/* operational mode constants -- used in SHM Configuration Table */
#define POLLED_MODE             0
#define INTR_MODE               1

/* error codes */

#define NO_ERROR                0
#define SHM_NO_FREE_PKTS        0xf0000

/* null pointers of different types */

#define NULL_ENV_CB             ((struct env_cb *) 0)
#define NULL_SHM_INFO           ((struct shm_info *) 0)
#define NULL_CONVERT            0
#if 0
#define NULL_CONVERT            (((rtems_unsigned32 *)())0) /* we want this */
#endif

/*  Node State Designators */

#define PENDING_INIT    Shm_convert(build_name( 'P', 'E', 'N', 'D' ))
#define INIT_COMPLETE   Shm_convert(build_name( 'C', 'O', 'M', 'P' ))
#define ACTIVE_NODE     Shm_convert(build_name( 'A', 'C', 'T', 'V' ))

/* The following is adjusted so envelopes are 0x80 bytes long. */
/* It should be >= MIN_PKT_SIZE in rtems.h                     */

#define MAX_PACKET_SIZE          (80)

/* constants pertinent to Locked Queue routines */

#define LQ_END_OF_LIST           Shm_convert( 0xffffffff )
#define LQ_NOT_ON_LIST           Shm_convert( 0xfffffffe )
#define LQ_UNLOCKED              0 

/* constants related to the Free Envelope Pool */

#define FREE_ENV_POOL            0 
#define FREE_ENV_CB              (&Lq_cbs[ FREE_ENV_POOL ])

/*  The following are important when dealing with 
 *  the shared memory communications interface area.
 *
 *  NOTE: The starting address and length of the shared memory
 *        is defined in a system dependent file.
 *
 *        MAX_ENVELOPES should be calculated to use as much of
 *        the remaining memory as possible.
 */
 
#define MAX_ENVELOPES    100
#define MAX_NODES        ( Shm_mp_config_tbl->maximum_nodes )

#define START_NS_CBS     ( (rtems_unsigned8 *) START_SHARED_MEM )
#define START_LQ_CBS     ( ((rtems_unsigned8 *) START_NS_CBS) + \
        ( (sizeof (struct node_status_cb)) * (MAX_NODES + 1) ) )
#define START_ENVELOPES  ( ((rtems_unsigned8 *) START_LQ_CBS) + \
        ( (sizeof (struct lockedq_cb)) * (MAX_NODES + 1) ) )
#define END_SHMCI_AREA    ( (rtems_unsigned8 *) START_ENVELOPES + \
        ( (sizeof (struct env_cb)) * MAX_ENVELOPES ) )
#define END_SHARED_MEM   ((rtems_unsigned32)START_SHARED_MEM+SHARED_MEM_LEN)

/* macros */

#define Is_master()             ( MASTER == Shm_local_node )
#define Local_node()            ( Shm_mp_config_tbl->node )
#define Set_local_intr_info( nscb, addr, val, len ) \
        { (nscb)->int_address = (rtems_unsigned32) (addr); \
          (nscb)->int_value   = (rtems_unsigned32) (val); \
          (nscb)->int_length  = (len); }

#define Free_envelope( ecb )    Add_lockedq( FREE_ENV_CB, (ecb) )
#define Get_envelope( ecb )     (ecb) = Del_lockedq(FREE_ENV_CB); 

#define Init_recvq(node)        Init_lockedq( &Lq_cbs[node], node )
#define Add_recvq(node, ecb)    Add_lockedq( &Lq_cbs[node], (ecb) )
#define Del_recvq(node)         Del_lockedq( &Lq_cbs[node] ); 

#define Ecb_to_pkt(ecb)         ((ecb)->packet)
#define Pkt_to_ecb( pkt )   \
   ((struct env_cb *)((rtems_unsigned8 *)(pkt) - \
   (sizeof(struct env_pre) + 4*sizeof(rtems_unsigned32))))

#define Build_preamble(ecb, node) \
       (ecb)->Preamble.endian = SHM_ENDIAN
#define Build_postamble( ecb ) 

/* structures */

/* shm control information */

struct shm_info {
  rtems_unsigned32 not_currently_used_0;
  rtems_unsigned32 not_currently_used_1;
  rtems_unsigned32 not_currently_used_2;
  rtems_unsigned32 not_currently_used_3;
};

struct env_pre {
  /*byte start_of_text;*/
  rtems_unsigned32 endian;
  rtems_unsigned32 not_currently_used_0;
  rtems_unsigned32 not_currently_used_1;
  rtems_unsigned32 not_currently_used_2;
};

struct env_post {
  rtems_unsigned32 not_currently_used_0;
  rtems_unsigned32 not_currently_used_1;
  rtems_unsigned32 not_currently_used_2;
  rtems_unsigned32 not_currently_used_3;
  /*byte end_of_text;*/
};

/* WARNING! If you change this structure, don't forget to change
 *          Ecb_to_pkt() and Pkt_to_ecb() above.
 */

/*  This comment block describes the contents of each field
 *  of the Envelope Control Block:
 *
 *  next      - The index of the next envelope on this queue.
 *  queue     - The index of the queue this envelope is on.
 *  index     - The index of this envelope.
 *  Preamble  - Generic packet preamble.  One day this structure
 *              could be enhanced to contain routing information.
 *  packet    - RTEMS MPCI packet.  Untouched by SHM Driver
 *              other than copying and format conversion as 
 *              documented in the RTEMS User's Guide.
 *  Postamble - Generic packet postamble.  One day this structure
 *              could be enhanced to contain checksum information.
 */

struct env_cb {
  rtems_unsigned32      next;     /* next envelope on queue       */
  rtems_unsigned32      queue;    /* queue on which this resides  */
  rtems_unsigned32      index;    /* index into array of envelopes*/
  rtems_unsigned32      pad0;     /* insure the next one is aligned */
  struct env_pre        Preamble; /* header information           */
  rtems_unsigned8       packet[MAX_PACKET_SIZE]; /* RTEMS INFO    */
  struct env_post       Postamble;/* trailer information          */
};

/*  This comment block describes the contents of each field
 *  of the Locked Queue Control Block:
 *
 *  lock      - Lock used to insure mutually exclusive access.
 *  front     - Index of first envelope on queue.  This field
 *              is used to remove head of queue (receive).
 *  rear      - Index of last envelope on queue.  This field
 *              is used to add evelope to queue (send).
 *  owner     - The node number of the recipient (owning) node.  
 *              RTEMS does not use the node number zero (0).
 *              The zero node is used by the SHM Driver for the
 *              Free Envelope Queue shared by all nodes.
 */

struct lockedq_cb {
  rtems_unsigned32 lock;          /* lock field for this queue    */
  rtems_unsigned32 front;         /* first envelope on queue      */
  rtems_unsigned32 rear;          /* last envelope on queue       */
  rtems_unsigned32 owner;         /* receiving (i.e. owning) node */
};

/*  This comment block describes the contents of each field
 *  of the Node Status Control Block:
 *
 *  status    - Node status.  Current values are PENDING_INIT,
 *              INIT_COMPLETE, and ACTIVE_NODE.  Other values
 *              could be added to enhance fault tolerance.
 *  error     - Zero if the node has not failed.  Otherwise,
 *              this field contains a status indicating the
 *              failure reason.
 *  int_address, int_value, and int_length
 *            - These field are the Interrupt Information table
 *              for this node in neutral format.  This is how 
 *              each node knows how to generate interrupts.
 */

struct node_status_cb {
  rtems_unsigned32  status;         /* node status information     */
  rtems_unsigned32  error;          /* fatal error code            */
  rtems_unsigned32  int_address;    /* write here for interrupt    */
  rtems_unsigned32  int_value;      /* this value causes interrupt */
  rtems_unsigned32  int_length;     /* for this length (0,1,2,4)   */
  rtems_unsigned32  not_currently_used_0;
  rtems_unsigned32  not_currently_used_1;
  rtems_unsigned32  not_currently_used_2;
};

/*  This comment block describes the contents of each field 
 *  of the Interrupt Information Table.  This table describes
 *  how another node can generate an interrupt to this node.
 *  This information is target board dependent.  If the 
 *  SHM Driver is in POLLED_MODE, then all fields should 
 *  be initialized to NO_INTERRUPT.
 * 
 *  address   - The address to which another node should 
 *              write to cause an interrupt.
 *  value     - The value which must be written
 *  length    - The size of the value to write.  Valid
 *              values are BYTE, WORD, and LONG.
 *
 *  NOTE:  The Node Status Control Block contains this 
 *         information in neutral format and not in a
 *         structure to avoid potential alignment problems.
 */
 
struct intr_info {
  rtems_unsigned32 *address;        /* write here for interrupt    */
  rtems_unsigned32  value;          /* this value causes interrupt */
  rtems_unsigned32  length;         /* for this length (0,1,2,4)   */
};

/*  SHM Configuration Table
 *
 *  This comment block describes the contents of each field 
 *  of the SHM Configuration Table.
 *
 *  base      - The base address of the shared memory.  This
 *              address may be specific to this node.
 *  length    - The length of the shared memory in bytes.
 *  format    - The natural format for rtems_unsigned32's in the 
 *              shared memory.  Valid values are currently 
 *              only SHM_LITTLE and SHM_BIG.
 *  convert   - The address of the routine which converts
 *              between neutral and local format.
 *  poll_intr - The operational mode of the driver.  Some 
 *              target boards may not provide hardware for 
 *              an interprocessor interrupt.  If POLLED_MODE
 *              is selected, the SHM driver will install a
 *              wrapper around the Clock_isr() to poll for
 *              incoming packets.  Throughput is dependent
 *              on the time between clock interrupts.
 *              Valid values are POLLED_MODE and INTR_MODE.
 *  Intr      - This structure describes the operation required
 *              to cause an interrupt to this node.  The actual
 *              contents of this structure are described above.
 */

struct shm_config_info {
  rtems_unsigned32       *base;     /* base address of SHM         */
  rtems_unsigned32        length;   /* length (in bytes) of SHM    */
  rtems_unsigned32        format;   /* SHM is big or little endian */
  rtems_unsigned32      (*convert)();/* neutral conversion routine */
  rtems_unsigned32        poll_intr;/* POLLED or INTR driven mode  */
  struct intr_info        Intr;     /* cause intr information      */
};

typedef struct shm_config_info shm_config_table;

/* global variables */

#ifdef _SHM_INIT
#define SHM_EXTERN
#else
#define SHM_EXTERN extern
#endif

SHM_EXTERN shm_config_table     *Shm_config;
SHM_EXTERN struct intr_info      Intr_info[16];
SHM_EXTERN struct node_status_cb *Ns_cbs;
SHM_EXTERN struct lockedq_cb     *Lq_cbs;
SHM_EXTERN struct env_cb         *Envelopes;
SHM_EXTERN rtems_configuration_table  *Shm_config_tbl;
SHM_EXTERN rtems_multiprocessing_table *Shm_mp_config_tbl;
SHM_EXTERN rtems_unsigned32             Shm_recvmsgcnt;
SHM_EXTERN rtems_unsigned32             Shm_nullmsgcnt;
SHM_EXTERN rtems_unsigned32             Shm_isr_count;
SHM_EXTERN rtems_unsigned32             Shm_local_node;
SHM_EXTERN struct lockedq_cb     *Local_lqcb;
SHM_EXTERN rtems_unsigned32             Shm_isrstat;
SHM_EXTERN rtems_unsigned32             Hi_idle, Lo_idle;  /* used by some tests */
                                                     /* reported by shmdr */
  
/* functions */

/* locked queue routines */
void           Add_lockedq( struct lockedq_cb *, struct env_cb * );
struct env_cb *Del_lockedq( struct lockedq_cb * );
void           Init_lockedq( struct lockedq_cb *, rtems_unsigned32 );
            /* Shm_lock is CPU dependent */
void           Shm_unlock( struct lockedq_cb * );

/* portable routines */
void           Init_env_pool();
void           Shm_cnv_pkt( rtems_unsigned8 * );
void           Shm_copy_mem();
rtems_task     Shm_dump( rtems_interval );
void           MPCI_fatal( rtems_unsigned32 );
rtems_task     Shm_idle();
rtems_task     Shm_interrupt( rtems_unsigned32 );
void           Shm_poll();
void           Shm_setclockvec();
void           Shm_cnvpkt( rtems_unsigned8 * );

/* CPU specific routines are inlined in shmcpu.h */

/* target specific routines */
rtems_unsigned32    *Shm_addrconv( rtems_unsigned32 * );
void           Shm_get_config( rtems_unsigned32, shm_config_table ** );
void           Shm_lock( struct lockedq_cb * );
void           Shm_isr();
void           Shm_setvec( void );

/* MPCI entry points */
void           Shm_getpkt( rtems_unsigned8 ** );
void           Shm_init( struct configuration_table * );
void           Shm_receive( rtems_unsigned8 ** );
void           Shm_retpkt( rtems_unsigned8 * );
void           Shm_send( rtems_unsigned32, rtems_unsigned8 *, rtems_unsigned32 );

#ifdef _SHM_INIT

/* multiprocessor communications interface (MPCI) table */

rtems_mpci_table MPCI_table  = {
  Shm_init,                   /* initialization procedure   */
  Shm_getpkt,                 /* get packet procedure       */
  Shm_retpkt,                 /* return packet procedure    */
  Shm_send,                   /* packet send procedure      */
  Shm_receive,                /* packet receive procedure   */
};

#else

extern rtems_mpci_table MPCI_table;

#endif

#endif
/* end of include file */
